"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Contravariant = exports.getMonoid = exports.getSemigroup = exports.EqStrict = exports.contramap = exports.tuple = exports.struct = exports.fromOrd = exports.fromEquals = void 0;
var function_1 = require("./function");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
var fromEquals = function (equals) { return ({
    equals: function (second) {
        var predicate = equals(second);
        return function (first) { return first === second || predicate(first); };
    }
}); };
exports.fromEquals = fromEquals;
/**
 * @category constructors
 * @since 3.0.0
 */
var fromOrd = function (O) { return exports.fromEquals(function (second) { return function (first) { return O.compare(second)(first) === 0; }; }); };
exports.fromOrd = fromOrd;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
var struct = function (eqs) {
    return exports.fromEquals(function (second) { return function (first) {
        for (var key in eqs) {
            if (!eqs[key].equals(second[key])(first[key])) {
                return false;
            }
        }
        return true;
    }; });
};
exports.struct = struct;
/**
 * Given a tuple of `Eq`s returns a `Eq` for the tuple
 *
 * @example
 * import { tuple } from 'fp-ts/Eq'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import * as B from 'fp-ts/boolean'
 *
 * const E = tuple(S.Eq, N.Eq, B.Eq)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, true]), true)
 * assert.strictEqual(E.equals(['a', 1, true])(['b', 1, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 2, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, false]), false)
 *
 * @category combinators
 * @since 3.0.0
 */
var tuple = function () {
    var eqs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        eqs[_i] = arguments[_i];
    }
    return exports.fromEquals(function (second) { return function (first) { return eqs.every(function (E, i) { return E.equals(second[i])(first[i]); }); }; });
};
exports.tuple = tuple;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Contravariant
 * @since 3.0.0
 */
var contramap = function (f) { return function (fa) {
    return exports.fromEquals(function (second) { return function_1.flow(f, fa.equals(f(second))); });
}; };
exports.contramap = contramap;
/**
 * @category instances
 * @since 3.0.0
 */
exports.EqStrict = {
    equals: function (second) { return function (first) { return first === second; }; }
};
/**
 * @category instances
 * @since 3.0.0
 */
var getSemigroup = function () { return ({
    concat: function (second) { return function (first) { return exports.fromEquals(function (b) { return function (a) { return first.equals(b)(a) && second.equals(b)(a); }; }); }; }
}); };
exports.getSemigroup = getSemigroup;
/**
 * @category instances
 * @since 3.0.0
 */
var getMonoid = function () { return ({
    concat: exports.getSemigroup().concat,
    empty: {
        equals: function () { return function () { return true; }; }
    }
}); };
exports.getMonoid = getMonoid;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Contravariant = {
    contramap: exports.contramap
};
