"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.between = exports.clamp = exports.max = exports.min = exports.geq = exports.leq = exports.gt = exports.lt = exports.equals = exports.trivial = exports.Contravariant = exports.getMonoid = exports.getSemigroup = exports.contramap = exports.reverse = exports.tuple = exports.fromCompare = void 0;
var function_1 = require("./function");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
var fromCompare = function (compare) { return ({
    compare: function (second) { return function (first) { return (first === second ? 0 : compare(second)(first)); }; }
}); };
exports.fromCompare = fromCompare;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Given a tuple of `Ord`s returns an `Ord` for the tuple.
 *
 * @example
 * import { tuple } from 'fp-ts/Ord'
 * import * as B from 'fp-ts/boolean'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const O = tuple(S.Ord, N.Ord, B.Ord)
 * assert.strictEqual(pipe(['a', 1, true], O.compare(['b', 2, true])), -1)
 * assert.strictEqual(pipe(['a', 1, true], O.compare(['a', 2, true])), -1)
 * assert.strictEqual(pipe(['a', 1, true], O.compare(['a', 1, false])), 1)
 *
 * @category combinators
 * @since 3.0.0
 */
var tuple = function () {
    var ords = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        ords[_i] = arguments[_i];
    }
    return exports.fromCompare(function (second) { return function (first) {
        var i = 0;
        for (; i < ords.length - 1; i++) {
            var r = ords[i].compare(second[i])(first[i]);
            if (r !== 0) {
                return r;
            }
        }
        return ords[i].compare(second[i])(first[i]);
    }; });
};
exports.tuple = tuple;
/**
 * @example
 * import { reverse } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, N.Ord.compare(6)), -1)
 * assert.deepStrictEqual(pipe(5, reverse(N.Ord).compare(6)), 1)
 *
 * @category combinators
 * @since 3.0.0
 */
var reverse = function (O) { return exports.fromCompare(function (second) { return function (first) { return O.compare(first)(second); }; }); };
exports.reverse = reverse;
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @example
 * import { contramap } from 'fp-ts/Ord'
 * import { sort } from 'fp-ts/ReadonlyArray'
 * import * as S from 'fp-ts/string'
 * import { pipe } from 'fp-ts/function'
 *
 * type User = {
 *   readonly name: string
 *   readonly age: number
 * }
 *
 * const byName = pipe(S.Ord, contramap((user: User) => user.name))
 *
 * const users: ReadonlyArray<User> = [
 *   { name: 'b', age: 1 },
 *   { name: 'a', age: 2 }
 * ]
 *
 * assert.deepStrictEqual(pipe(users, sort(byName)), [
 *   { name: 'a', age: 2 },
 *   { name: 'b', age: 1 }
 * ])
 *
 * @category Contravariant
 * @since 3.0.0
 */
var contramap = function (f) { return function (fa) {
    return exports.fromCompare(function (second) { return function (first) { return fa.compare(f(second))(f(first)); }; });
}; };
exports.contramap = contramap;
/**
 * Returns a `Semigroup` such that:
 *
 * - `pipe(ord1, concat(ord2))` will order first by `ord1`, and then by `ord2`
 *
 * @category instances
 * @since 3.0.0
 */
var getSemigroup = function () { return ({
    concat: function (second) { return function (first) {
        return exports.fromCompare(function (a2) { return function (a1) {
            var ox = first.compare(a2)(a1);
            return ox !== 0 ? ox : second.compare(a2)(a1);
        }; });
    }; }
}); };
exports.getSemigroup = getSemigroup;
/**
 * Returns a `Monoid` such that:
 *
 * - `pipe(ord1, concat(ord2))` will order first by `ord1`, and then by `ord2`
 * - its `empty` value is an `Ord` that always considers compared elements equal
 *
 * @example
 * import { sort } from 'fp-ts/ReadonlyArray'
 * import { contramap, reverse, getMonoid } from 'fp-ts/Ord'
 * import { pipe } from 'fp-ts/function'
 * import { concatAll } from 'fp-ts/Monoid'
 * import * as B from 'fp-ts/boolean'
 * import * as N from 'fp-ts/number'
 * import * as S from 'fp-ts/string'
 *
 * interface User {
 *   id: number
 *   name: string
 *   age: number
 *   rememberMe: boolean
 * }
 *
 * const byName = pipe(
 *   S.Ord,
 *   contramap((p: User) => p.name)
 * )
 *
 * const byAge = pipe(
 *   N.Ord,
 *   contramap((p: User) => p.age)
 * )
 *
 * const byRememberMe = pipe(
 *   B.Ord,
 *   contramap((p: User) => p.rememberMe)
 * )
 *
 * const M = getMonoid<User>()
 *
 * const users: ReadonlyArray<User> = [
 *   { id: 1, name: 'Guido', age: 47, rememberMe: false },
 *   { id: 2, name: 'Guido', age: 46, rememberMe: true },
 *   { id: 3, name: 'Giulio', age: 44, rememberMe: false },
 *   { id: 4, name: 'Giulio', age: 44, rememberMe: true }
 * ]
 *
 * // sort by name, then by age, then by `rememberMe`
 * const O1 = concatAll(M)([byName, byAge, byRememberMe])
 * assert.deepStrictEqual(sort(O1)(users), [
 *   { id: 3, name: 'Giulio', age: 44, rememberMe: false },
 *   { id: 4, name: 'Giulio', age: 44, rememberMe: true },
 *   { id: 2, name: 'Guido', age: 46, rememberMe: true },
 *   { id: 1, name: 'Guido', age: 47, rememberMe: false }
 * ])
 *
 * // now `rememberMe = true` first, then by name, then by age
 * const O2 = concatAll(M)([reverse(byRememberMe), byName, byAge])
 * assert.deepStrictEqual(sort(O2)(users), [
 *   { id: 4, name: 'Giulio', age: 44, rememberMe: true },
 *   { id: 2, name: 'Guido', age: 46, rememberMe: true },
 *   { id: 3, name: 'Giulio', age: 44, rememberMe: false },
 *   { id: 1, name: 'Guido', age: 47, rememberMe: false }
 * ])
 *
 * @category instances
 * @since 3.0.0
 */
var getMonoid = function () { return ({
    concat: exports.getSemigroup().concat,
    empty: exports.fromCompare(function () { return function () { return 0; }; })
}); };
exports.getMonoid = getMonoid;
/**
 * @category instances
 * @since 3.0.0
 */
exports.Contravariant = {
    contramap: exports.contramap
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
exports.trivial = {
    compare: function () { return function () { return 0; }; }
};
/**
 * @since 3.0.0
 */
var equals = function (O) { return function (second) { return function (first) {
    return first === second || O.compare(second)(first) === 0;
}; }; };
exports.equals = equals;
/**
 * Test whether one value is _strictly less than_ another.
 *
 * @example
 * import { lt } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, lt(N.Ord)(4)), false)
 * assert.deepStrictEqual(pipe(5, lt(N.Ord)(5)), false)
 * assert.deepStrictEqual(pipe(5, lt(N.Ord)(6)), true)
 *
 * @since 3.0.0
 */
var lt = function (O) { return function (second) { return function (first) { return O.compare(second)(first) === -1; }; }; };
exports.lt = lt;
/**
 * Test whether one value is _strictly greater than_ another.
 *
 * @example
 * import { gt } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, gt(N.Ord)(4)), true)
 * assert.deepStrictEqual(pipe(5, gt(N.Ord)(5)), false)
 * assert.deepStrictEqual(pipe(5, gt(N.Ord)(6)), false)
 *
 * @since 3.0.0
 */
var gt = function (O) { return function (second) { return function (first) { return O.compare(second)(first) === 1; }; }; };
exports.gt = gt;
/**
 * Test whether one value is _non-strictly less than_ another.
 *
 * @example
 * import { leq } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, leq(N.Ord)(4)), false)
 * assert.deepStrictEqual(pipe(5, leq(N.Ord)(5)), true)
 * assert.deepStrictEqual(pipe(5, leq(N.Ord)(6)), true)
 *
 * @since 3.0.0
 */
var leq = function (O) { return function (second) { return function (first) { return O.compare(second)(first) !== 1; }; }; };
exports.leq = leq;
/**
 * Test whether one value is _non-strictly greater than_ another.
 *
 * @example
 * import { geq } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, geq(N.Ord)(4)), true)
 * assert.deepStrictEqual(pipe(5, geq(N.Ord)(5)), true)
 * assert.deepStrictEqual(pipe(5, geq(N.Ord)(6)), false)
 *
 * @since 3.0.0
 */
var geq = function (O) { return function (second) { return function (first) { return O.compare(second)(first) !== -1; }; }; };
exports.geq = geq;
/**
 * Take the minimum of two values. If they are considered equal, the first argument is chosen.
 *
 * @example
 * import { min } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, min(N.Ord)(6)), 5)
 *
 * @since 3.0.0
 */
var min = function (O) { return function (second) { return function (first) {
    return first === second || O.compare(second)(first) < 1 ? first : second;
}; }; };
exports.min = min;
/**
 * Take the maximum of two values. If they are considered equal, the first argument is chosen.
 *
 * @example
 * import { max } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(5, max(N.Ord)(6)), 6)
 *
 * @since 3.0.0
 */
var max = function (O) { return function (second) { return function (first) {
    return first === second || O.compare(second)(first) > -1 ? first : second;
}; }; };
exports.max = max;
/**
 * Clamp a value between a minimum and a maximum.
 *
 * @example
 * import { clamp } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const f = clamp(N.Ord)(2, 4)
 * assert.deepStrictEqual(pipe(1, f), 2)
 * assert.deepStrictEqual(pipe(3, f), 3)
 * assert.deepStrictEqual(pipe(5, f), 4)
 *
 * @since 3.0.0
 */
var clamp = function (O) {
    var minO = exports.min(O);
    var maxO = exports.max(O);
    return function (low, hi) { return function_1.flow(minO(hi), maxO(low)); };
};
exports.clamp = clamp;
/**
 * Test whether a value is between a minimum and a maximum (inclusive).
 *
 * @example
 * import { between } from 'fp-ts/Ord'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const f = between(N.Ord)(2, 4)
 * assert.deepStrictEqual(pipe(1, f), false)
 * assert.deepStrictEqual(pipe(3, f), true)
 * assert.deepStrictEqual(pipe(5, f), false)
 *
 * @since 3.0.0
 */
var between = function (O) {
    var ltO = exports.lt(O);
    var gtO = exports.gt(O);
    return function (low, hi) { return function (a) { return (ltO(low)(a) || gtO(hi)(a) ? false : true); }; };
};
exports.between = between;
