"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.negate = exports.tuple = void 0;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Given a tuple of `Ring`s returns a `Ring` for the tuple
 *
 * @example
 * import { tuple } from 'fp-ts/Ring'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const R = tuple(N.Field, N.Field, N.Field)
 * assert.deepStrictEqual(pipe([1, 2, 3], R.add([4, 5, 6])), [5, 7, 9])
 * assert.deepStrictEqual(pipe([1, 2, 3], R.mul([4, 5, 6])), [4, 10, 18])
 * assert.deepStrictEqual(R.one, [1, 1, 1])
 * assert.deepStrictEqual(pipe([1, 2, 3], R.sub([4, 5, 6])), [-3, -3, -3])
 * assert.deepStrictEqual(R.zero, [0, 0, 0])
 *
 * @category combinators
 * @since 3.0.0
 */
var tuple = function () {
    var rings = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        rings[_i] = arguments[_i];
    }
    return ({
        add: function (second) { return function (first) { return rings.map(function (R, i) { return R.add(second[i])(first[i]); }); }; },
        zero: rings.map(function (R) { return R.zero; }),
        mul: function (second) { return function (first) { return rings.map(function (R, i) { return R.mul(second[i])(first[i]); }); }; },
        one: rings.map(function (R) { return R.one; }),
        sub: function (second) { return function (first) { return rings.map(function (R, i) { return R.sub(second[i])(first[i]); }); }; }
    });
};
exports.tuple = tuple;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * `negate x` can be used as a shorthand for `zero - x`
 *
 * @since 3.0.0
 */
var negate = function (ring) { return function (a) { return ring.sub(a)(ring.zero); }; };
exports.negate = negate;
