"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SK = exports.hole = exports.pipe = exports.untupled = exports.tupled = exports.absurd = exports.decrement = exports.increment = exports.flow = exports.flip = exports.constVoid = exports.constUndefined = exports.constNull = exports.constFalse = exports.constTrue = exports.constant = exports.unsafeCoerce = exports.identity = exports.apply = exports.getRing = exports.getSemiring = exports.getMonoid = exports.getSemigroup = exports.getBooleanAlgebra = void 0;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
var getBooleanAlgebra = function (BA) { return function () { return ({
    meet: function (second) { return function (first) { return function (a) { return BA.meet(second(a))(first(a)); }; }; },
    join: function (second) { return function (first) { return function (a) { return BA.join(second(a))(first(a)); }; }; },
    zero: function () { return BA.zero; },
    one: function () { return BA.one; },
    implies: function (second) { return function (first) { return function (a) { return BA.implies(second(a))(first(a)); }; }; },
    not: function (x) { return function (a) { return BA.not(x(a)); }; }
}); }; };
exports.getBooleanAlgebra = getBooleanAlgebra;
/**
 * Unary functions form a semigroup as long as you can provide a semigroup for the codomain.
 *
 * @example
 * import { Predicate } from 'fp-ts/Predicate'
 * import { pipe, getSemigroup } from 'fp-ts/function'
 * import * as B from 'fp-ts/boolean'
 *
 * const f: Predicate<number> = (n) => n <= 2
 * const g: Predicate<number> = (n) => n >= 0
 *
 * const S1 = getSemigroup(B.SemigroupAll)<number>()
 *
 * assert.deepStrictEqual(pipe(f, S1.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, S1.concat(g))(3), false)
 *
 * const S2 = getSemigroup(B.SemigroupAny)<number>()
 *
 * assert.deepStrictEqual(pipe(f, S2.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, S2.concat(g))(3), true)
 *
 * @category instances
 * @since 3.0.0
 */
var getSemigroup = function (S) { return function () { return ({
    concat: function (second) { return function (first) { return function (a) { return S.concat(second(a))(first(a)); }; }; }
}); }; };
exports.getSemigroup = getSemigroup;
/**
 * Unary functions form a monoid as long as you can provide a monoid for the codomain.
 *
 * @example
 * import { Predicate } from 'fp-ts/Predicate'
 * import { getMonoid, pipe } from 'fp-ts/function'
 * import * as B from 'fp-ts/boolean'
 *
 * const f: Predicate<number> = (n) => n <= 2
 * const g: Predicate<number> = (n) => n >= 0
 *
 * const M1 = getMonoid(B.MonoidAll)<number>()
 *
 * assert.deepStrictEqual(pipe(f, M1.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, M1.concat(g))(3), false)
 *
 * const M2 = getMonoid(B.MonoidAny)<number>()
 *
 * assert.deepStrictEqual(pipe(f, M2.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, M2.concat(g))(3), true)
 *
 * @category instances
 * @since 3.0.0
 */
var getMonoid = function (M) {
    var getSemigroupM = exports.getSemigroup(M);
    return function () { return ({
        concat: getSemigroupM().concat,
        empty: function () { return M.empty; }
    }); };
};
exports.getMonoid = getMonoid;
/**
 * @category instances
 * @since 3.0.0
 */
var getSemiring = function (S) { return ({
    add: function (second) { return function (first) { return function (x) { return S.add(second(x))(first(x)); }; }; },
    zero: function () { return S.zero; },
    mul: function (second) { return function (first) { return function (x) { return S.mul(second(x))(first(x)); }; }; },
    one: function () { return S.one; }
}); };
exports.getSemiring = getSemiring;
/**
 * @category instances
 * @since 3.0.0
 */
var getRing = function (R) {
    var S = exports.getSemiring(R);
    return {
        add: S.add,
        mul: S.mul,
        one: S.one,
        zero: S.zero,
        sub: function (second) { return function (first) { return function (x) { return R.sub(second(x))(first(x)); }; }; }
    };
};
exports.getRing = getRing;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
var apply = function (a) { return function (f) { return f(a); }; };
exports.apply = apply;
/**
 * @since 3.0.0
 */
var identity = function (a) { return a; };
exports.identity = identity;
/**
 * @since 3.0.0
 */
exports.unsafeCoerce = exports.identity;
/**
 * @since 3.0.0
 */
var constant = function (a) { return function () { return a; }; };
exports.constant = constant;
/**
 * A thunk that returns always `true`.
 *
 * @since 3.0.0
 */
exports.constTrue = 
/*#__PURE__*/
exports.constant(true);
/**
 * A thunk that returns always `false`.
 *
 * @since 3.0.0
 */
exports.constFalse = 
/*#__PURE__*/
exports.constant(false);
/**
 * A thunk that returns always `null`.
 *
 * @since 3.0.0
 */
exports.constNull = 
/*#__PURE__*/
exports.constant(null);
/**
 * A thunk that returns always `undefined`.
 *
 * @since 3.0.0
 */
exports.constUndefined = 
/*#__PURE__*/
exports.constant(undefined);
/**
 * A thunk that returns always `void`.
 *
 * @since 3.0.0
 */
exports.constVoid = exports.constUndefined;
/**
 * Flips the order of the arguments of a function of two arguments.
 *
 * @since 3.0.0
 */
var flip = function (f) { return function (b, a) { return f(a, b); }; };
exports.flip = flip;
function flow(ab, bc, cd, de, ef, fg, gh, hi, ij) {
    switch (arguments.length) {
        case 1:
            return ab;
        case 2:
            return function () {
                return bc(ab.apply(this, arguments));
            };
        case 3:
            return function () {
                return cd(bc(ab.apply(this, arguments)));
            };
        case 4:
            return function () {
                return de(cd(bc(ab.apply(this, arguments))));
            };
        case 5:
            return function () {
                return ef(de(cd(bc(ab.apply(this, arguments)))));
            };
        case 6:
            return function () {
                return fg(ef(de(cd(bc(ab.apply(this, arguments))))));
            };
        case 7:
            return function () {
                return gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))));
            };
        case 8:
            return function () {
                return hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments))))))));
            };
        case 9:
            return function () {
                return ij(hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))))));
            };
    }
    return;
}
exports.flow = flow;
/**
 * @since 3.0.0
 */
var increment = function (n) { return n + 1; };
exports.increment = increment;
/**
 * @since 3.0.0
 */
var decrement = function (n) { return n - 1; };
exports.decrement = decrement;
/**
 * @since 3.0.0
 */
var absurd = function (_) {
    throw new Error('Called `absurd` function which should be uncallable');
};
exports.absurd = absurd;
/**
 * Creates a tupled version of this function: instead of `n` arguments, it accepts a single tuple argument.
 *
 * @example
 * import { tupled } from 'fp-ts/function'
 *
 * const add = tupled((x: number, y: number): number => x + y)
 *
 * assert.strictEqual(add([1, 2]), 3)
 *
 * @since 3.0.0
 */
var tupled = function (f) { return function (a) { return f.apply(void 0, a); }; };
exports.tupled = tupled;
/**
 * Inverse function of `tupled`
 *
 * @since 3.0.0
 */
var untupled = function (f) { return function () {
    var a = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        a[_i] = arguments[_i];
    }
    return f(a);
}; };
exports.untupled = untupled;
function pipe(a, ab, bc, cd, de, ef, fg, gh, hi) {
    switch (arguments.length) {
        case 1:
            return a;
        case 2:
            return ab(a);
        case 3:
            return bc(ab(a));
        case 4:
            return cd(bc(ab(a)));
        case 5:
            return de(cd(bc(ab(a))));
        case 6:
            return ef(de(cd(bc(ab(a)))));
        case 7:
            return fg(ef(de(cd(bc(ab(a))))));
        case 8:
            return gh(fg(ef(de(cd(bc(ab(a)))))));
        case 9:
            return hi(gh(fg(ef(de(cd(bc(ab(a))))))));
        default:
            var ret = arguments[0];
            for (var i = 1; i < arguments.length; i++) {
                ret = arguments[i](ret);
            }
            return ret;
    }
}
exports.pipe = pipe;
/**
 * Type hole simulation
 *
 * @since 3.0.0
 */
exports.hole = exports.absurd;
/**
 * `SK` function (SKI combinator calculus).
 *
 * @since 3.0.0
 */
var SK = function (_, b) { return b; };
exports.SK = SK;
