import { flow } from './function';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromEquals = (equals) => ({
    equals: (second) => {
        const predicate = equals(second);
        return (first) => first === second || predicate(first);
    }
});
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromOrd = (O) => fromEquals((second) => (first) => O.compare(second)(first) === 0);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const struct = (eqs) => fromEquals((second) => (first) => {
    for (const key in eqs) {
        if (!eqs[key].equals(second[key])(first[key])) {
            return false;
        }
    }
    return true;
});
/**
 * Given a tuple of `Eq`s returns a `Eq` for the tuple
 *
 * @example
 * import { tuple } from 'fp-ts/Eq'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import * as B from 'fp-ts/boolean'
 *
 * const E = tuple(S.Eq, N.Eq, B.Eq)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, true]), true)
 * assert.strictEqual(E.equals(['a', 1, true])(['b', 1, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 2, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, false]), false)
 *
 * @category combinators
 * @since 3.0.0
 */
export const tuple = (...eqs) => fromEquals((second) => (first) => eqs.every((E, i) => E.equals(second[i])(first[i])));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Contravariant
 * @since 3.0.0
 */
export const contramap = (f) => (fa) => fromEquals((second) => flow(f, fa.equals(f(second))));
/**
 * @category instances
 * @since 3.0.0
 */
export const EqStrict = {
    equals: (second) => (first) => first === second
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = () => ({
    concat: (second) => (first) => fromEquals((b) => (a) => first.equals(b)(a) && second.equals(b)(a))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = () => ({
    concat: getSemigroup().concat,
    empty: {
        equals: () => () => true
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Contravariant = {
    contramap
};
