import { fromEquals } from './Eq';
import { flow, pipe } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
import * as O from './Option';
import { separated } from './Separated';
import { snd } from './Tuple2';
import { wiltDefault, witherDefault } from './Witherable';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Create a `ReadonlyMap` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export const singleton = (k, a) => new Map([[k, a]]);
export function fromFoldable(F) {
    return (E, M) => {
        const lookupWithKeyE = lookupWithKey(E);
        return (f) => F.reduce(new Map(), (out, a) => {
            const [k, b] = f(a);
            const oka = lookupWithKeyE(k)(out);
            if (_.isSome(oka)) {
                out.set(oka.value[0], M.concat(b)(oka.value[1]));
            }
            else {
                out.set(k, b);
            }
            return out;
        });
    };
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Insert an element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export const insertAt = (E) => {
    const memberE = member(E);
    const upsertAtE = upsertAt(E);
    return (k, a) => {
        const memberEk = memberE(k);
        const upsertAtEka = upsertAtE(k, a);
        return (m) => (memberEk(m) ? _.none : _.some(upsertAtEka(m)));
    };
};
/**
 * Insert or replace a key/value pair in a `ReadonlyMap`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const upsertAt = (E) => {
    const lookupWithKeyE = lookupWithKey(E);
    return (k, a) => {
        const lookupWithKeyEk = lookupWithKeyE(k);
        return (m) => {
            const found = lookupWithKeyEk(m);
            if (_.isNone(found)) {
                const out = new Map(m);
                out.set(k, a);
                return out;
            }
            else if (found.value[1] !== a) {
                const out = new Map(m);
                out.set(found.value[0], a);
                return out;
            }
            return m;
        };
    };
};
/**
 * Change the element at the specified keys, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export const updateAt = (E) => {
    const modifyAtE = modifyAt(E);
    return (k, a) => modifyAtE(k, () => a);
};
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export const modifyAt = (E) => {
    const lookupWithKeyE = lookupWithKey(E);
    return (k, f) => {
        const lookupWithKeyEk = lookupWithKeyE(k);
        return (m) => {
            const found = lookupWithKeyEk(m);
            if (_.isNone(found)) {
                return _.none;
            }
            const [fk, fv] = found.value;
            const next = f(fv);
            if (next === fv) {
                return _.some(m);
            }
            const r = new Map(m);
            r.set(fk, next);
            return _.some(r);
        };
    };
};
/**
 * Delete the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const deleteAt = (E) => {
    const popE = pop(E);
    return (k) => flow(popE(k), O.map(snd));
};
/**
 * Delete a key and value from a `ReadonlyMap`, returning the value as well as the subsequent `ReadonlyMap`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const pop = (E) => {
    const lookupWithKeyE = lookupWithKey(E);
    return (k) => {
        const lookupWithKeyEk = lookupWithKeyE(k);
        return (m) => {
            const found = lookupWithKeyEk(m);
            return pipe(found, O.map(([k, a]) => {
                const out = new Map(m);
                out.delete(k);
                return [a, out];
            }));
        };
    };
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => mapWithIndex((_, a) => f(a));
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export const mapWithIndex = (f) => (m) => {
    const out = new Map();
    const entries = m.entries();
    let e;
    while (!(e = entries.next()).done) {
        const [key, a] = e.value;
        out.set(key, f(key, a));
    }
    return out;
};
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = (m) => {
    const out = new Map();
    const entries = m.entries();
    let e;
    while (!(e = entries.next()).done) {
        const [k, oa] = e.value;
        if (_.isSome(oa)) {
            out.set(k, oa.value);
        }
    }
    return out;
};
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = (fa) => {
    const left = new Map();
    const right = new Map();
    const entries = fa.entries();
    let e;
    while (!(e = entries.next()).done) {
        const [k, ei] = e.value;
        if (_.isLeft(ei)) {
            left.set(k, ei.left);
        }
        else {
            right.set(k, ei.right);
        }
    }
    return separated(left, right);
};
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filter = (predicate) => (fa) => pipe(fa, filterWithIndex((_, a) => predicate(a)));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = (f) => filterMapWithIndex((_, a) => f(a));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partition = (predicate) => (fa) => pipe(fa, partitionWithIndex((_, a) => predicate(a)));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = (f) => partitionMapWithIndex((_, a) => f(a));
export function filterWithIndex(predicateWithIndex) {
    return (m) => {
        const out = new Map();
        const entries = m.entries();
        let e;
        // tslint:disable-next-line: strict-boolean-expressions
        while (!(e = entries.next()).done) {
            const [k, a] = e.value;
            if (predicateWithIndex(k, a)) {
                out.set(k, a);
            }
        }
        return out;
    };
}
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export const filterMapWithIndex = (f) => (fa) => {
    const m = new Map();
    const entries = fa.entries();
    let e;
    while (!(e = entries.next()).done) {
        const [k, a] = e.value;
        const o = f(k, a);
        if (_.isSome(o)) {
            m.set(k, o.value);
        }
    }
    return m;
};
export function partitionWithIndex(predicateWithIndex) {
    return (m) => {
        const left = new Map();
        const right = new Map();
        const entries = m.entries();
        let e;
        // tslint:disable-next-line: strict-boolean-expressions
        while (!(e = entries.next()).done) {
            const [k, a] = e.value;
            if (predicateWithIndex(k, a)) {
                right.set(k, a);
            }
            else {
                left.set(k, a);
            }
        }
        return separated(left, right);
    };
}
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export const partitionMapWithIndex = (f) => (fa) => {
    const left = new Map();
    const right = new Map();
    const entries = fa.entries();
    let e;
    while (!(e = entries.next()).done) {
        const [k, a] = e.value;
        const ei = f(k, a);
        if (_.isLeft(ei)) {
            left.set(k, ei.left);
        }
        else {
            right.set(k, ei.right);
        }
    }
    return separated(left, right);
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (SK, SA) => ({
    show: (m) => {
        const entries = [];
        m.forEach((a, k) => {
            entries.push(`[${SK.show(k)}, ${SA.show(a)}]`);
        });
        return `new Map([${entries.sort().join(', ')}])`;
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (EK, EA) => {
    const isSubmapSKSA = isSubmap(EK, EA);
    return fromEquals((second) => (first) => isSubmapSKSA(first)(second) && isSubmapSKSA(second)(first));
};
/**
 * Get a `Monoid` instance for `ReadonlyMap` given a `Semigroup` instance for its values.
 *
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = (EK, SA) => {
    const lookupWithKeyS = lookupWithKey(EK);
    return {
        concat: (second) => (first) => {
            if (isEmpty(first)) {
                return second;
            }
            if (isEmpty(second)) {
                return first;
            }
            const r = new Map(first);
            const entries = second.entries();
            let e;
            while (!(e = entries.next()).done) {
                const [k, a] = e.value;
                const oka = lookupWithKeyS(k)(first);
                if (_.isSome(oka)) {
                    r.set(oka.value[0], SA.concat(a)(oka.value[1]));
                }
                else {
                    r.set(k, a);
                }
            }
            return r;
        },
        empty
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const getFunctorWithIndex = () => ({
    mapWithIndex
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    partitionMap,
    partition,
    filterMap,
    filter
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterableWithIndex = () => ({
    filterWithIndex,
    filterMapWithIndex,
    partitionWithIndex,
    partitionMapWithIndex
});
/**
 * @since 3.0.0
 */
export const reduce = (O) => {
    const reduceWithIndexO = reduceWithIndex(O);
    return (b, f) => reduceWithIndexO(b, (_, b, a) => f(b, a));
};
/**
 * @since 3.0.0
 */
export const foldMap = (O) => {
    const foldMapWithIndexO = foldMapWithIndex(O);
    return (M) => {
        const foldMapWithIndexOM = foldMapWithIndexO(M);
        return (f) => foldMapWithIndexOM((_, a) => f(a));
    };
};
/**
 * @since 3.0.0
 */
export const reduceRight = (O) => {
    const reduceRightWithIndexO = reduceRightWithIndex(O);
    return (b, f) => reduceRightWithIndexO(b, (_, b, a) => f(b, a));
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFoldable = (O) => {
    return {
        reduce: reduce(O),
        foldMap: foldMap(O),
        reduceRight: reduceRight(O)
    };
};
/**
 * @since 3.0.0
 */
export const reduceWithIndex = (O) => {
    const keysO = keys(O);
    return (b, f) => (m) => {
        let out = b;
        for (const k of keysO(m)) {
            out = f(k, out, m.get(k));
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const foldMapWithIndex = (O) => {
    const keysO = keys(O);
    return (M) => (f) => (m) => {
        let out = M.empty;
        for (const k of keysO(m)) {
            out = M.concat(f(k, m.get(k)))(out);
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const reduceRightWithIndex = (O) => {
    const keysO = keys(O);
    return (b, f) => (m) => {
        let out = b;
        const ks = keysO(m);
        const len = ks.length;
        for (let i = len - 1; i >= 0; i--) {
            const k = ks[i];
            out = f(k, m.get(k), out);
        }
        return out;
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFoldableWithIndex = (O) => {
    return {
        reduceWithIndex: reduceWithIndex(O),
        foldMapWithIndex: foldMapWithIndex(O),
        reduceRightWithIndex: reduceRightWithIndex(O)
    };
};
/**
 * @since 3.0.0
 */
export const traverse = (O) => {
    const traverseWithIndexO = traverseWithIndex(O);
    return (F) => {
        const traverseWithIndexOF = traverseWithIndexO(F);
        return (f) => traverseWithIndexOF((_, a) => f(a));
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getTraversable = (O) => {
    return {
        map,
        traverse: traverse(O)
    };
};
/**
 * @since 3.0.0
 */
export const traverseWithIndex = (O) => {
    const keysO = keys(O);
    return (F) => (f) => (ta) => {
        let fm = F.of(new Map());
        for (const k of keysO(ta)) {
            const a = ta.get(k);
            fm = pipe(fm, F.map((m) => (b) => m.set(k, b)), F.ap(f(k, a)));
        }
        return fm;
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getTraversableWithIndex = (O) => {
    return {
        traverseWithIndex: traverseWithIndex(O)
    };
};
/**
 * @since 3.0.0
 */
export const wither = (O) => {
    return witherDefault(getTraversable(O), Compactable);
};
/**
 * @since 3.0.0
 */
export const wilt = (O) => {
    return wiltDefault(getTraversable(O), Compactable);
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getWitherable = (O) => {
    return {
        wither: wither(O),
        wilt: wilt(O)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionSemigroup = (E, S) => ({
    concat: union(E, S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionMonoid = (E, S) => ({
    concat: getUnionSemigroup(E, S).concat,
    empty
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getIntersectionSemigroup = (E, S) => ({
    concat: intersection(E, S)
});
/**
 * @category combinator
 * @since 3.0.0
 */
export const getDifferenceMagma = (E) => () => ({
    concat: difference(E)
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Calculate the number of key/value pairs in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const size = (d) => d.size;
/**
 * Test whether or not a `ReadonlyMap` is empty.
 *
 * @since 3.0.0
 */
export const isEmpty = (d) => d.size === 0;
/**
 * Test whether or not a key exists in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const member = (E) => {
    const lookupE = lookup(E);
    return (k) => {
        const lookupEk = lookupE(k);
        return (m) => _.isSome(lookupEk(m));
    };
};
/**
 * Tests whether a value is a member of a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => {
    const predicate = E.equals(a);
    return (m) => {
        const values = m.values();
        let e;
        while (!(e = values.next()).done) {
            const v = e.value;
            if (predicate(v)) {
                return true;
            }
        }
        return false;
    };
};
/**
 * Get a sorted `ReadonlyArray` of the keys contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const keys = (O) => (m) => Array.from(m.keys()).sort((first, second) => O.compare(second)(first));
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const values = (O) => (m) => Array.from(m.values()).sort((first, second) => O.compare(second)(first));
/**
 * @since 3.0.0
 */
export const collect = (O) => {
    const keysO = keys(O);
    return (f) => (m) => {
        const out = [];
        const ks = keysO(m);
        for (const key of ks) {
            out.push(f(key, m.get(key)));
        }
        return out;
    };
};
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 * If the result is a `Some`, the existing key is also returned.
 *
 * @since 3.0.0
 */
export const lookupWithKey = (E) => (k) => {
    const predicate = E.equals(k);
    return (m) => {
        const entries = m.entries();
        let e;
        while (!(e = entries.next()).done) {
            const [k, a] = e.value;
            if (predicate(k)) {
                return _.some([k, a]);
            }
        }
        return _.none;
    };
};
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const lookup = (E) => {
    const lookupWithKeyE = lookupWithKey(E);
    return (k) => flow(lookupWithKeyE(k), O.map(snd));
};
/**
 * Test whether or not one `ReadonlyMap` contains all of the keys and values contained in another `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const isSubmap = (EK, SA) => {
    const lookupWithKeyS = lookupWithKey(EK);
    return (second) => (first) => {
        const entries = first.entries();
        let e;
        while (!(e = entries.next()).done) {
            const [k, a] = e.value;
            const oka = lookupWithKeyS(k)(second);
            if (_.isNone(oka) || !EK.equals(oka.value[0])(k) || !SA.equals(oka.value[1])(a)) {
                return false;
            }
        }
        return true;
    };
};
/**
 * An empty `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const empty = new Map();
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export const toReadonlyArray = (O) => collect(O)((k, a) => [k, a]);
export function toUnfoldable(U) {
    return (o) => {
        const toReadonlyArrayO = toReadonlyArray(o);
        return (m) => {
            const arr = toReadonlyArrayO(m);
            const len = arr.length;
            return U.unfold(0, (b) => (b < len ? _.some([arr[b], b + 1]) : _.none));
        };
    };
}
/**
 * @since 3.0.0
 */
export const union = (E, M) => {
    const lookupE = lookup(E);
    return (second) => (first) => {
        if (isEmpty(first)) {
            return second;
        }
        if (isEmpty(second)) {
            return first;
        }
        const out = new Map();
        const firstEntries = first.entries();
        let e;
        while (!(e = firstEntries.next()).done) {
            const [k, a] = e.value;
            const oka = lookupE(k)(second);
            if (_.isSome(oka)) {
                out.set(k, M.concat(oka.value)(a));
            }
            else {
                out.set(k, a);
            }
        }
        const secondEntries = second.entries();
        while (!(e = secondEntries.next()).done) {
            const [k, a] = e.value;
            const oka = lookupE(k)(out);
            if (_.isNone(oka)) {
                out.set(k, a);
            }
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const intersection = (E, M) => {
    const lookupE = lookup(E);
    return (second) => (first) => {
        if (isEmpty(first) || isEmpty(second)) {
            return empty;
        }
        const out = new Map();
        const entries = first.entries();
        let e;
        while (!(e = entries.next()).done) {
            const [k, a] = e.value;
            const oka = lookupE(k)(second);
            if (_.isSome(oka)) {
                out.set(k, M.concat(oka.value)(a));
            }
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const difference = (E) => {
    const memberE = member(E);
    return (second) => (first) => {
        if (isEmpty(first)) {
            return second;
        }
        if (isEmpty(second)) {
            return first;
        }
        const out = new Map();
        const firstEntries = first.entries();
        let e;
        while (!(e = firstEntries.next()).done) {
            const [k, a] = e.value;
            if (!memberE(k)(second)) {
                out.set(k, a);
            }
        }
        const secondEntries = second.entries();
        while (!(e = secondEntries.next()).done) {
            const [k, a] = e.value;
            if (!memberE(k)(first)) {
                out.set(k, a);
            }
        }
        return out;
    };
};
