import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { bind as bind_, chainFirst as chainFirst_ } from './Chain';
import { fromEquals } from './Eq';
import { flow, identity, pipe } from './function';
import { bindTo as bindTo_, flap as flap_, tupled as tupled_ } from './Functor';
import * as _ from './internal';
import { fromReadonlyNonEmptyArray } from './NonEmptyArray';
import { getMonoid } from './Ord';
import * as Se from './Semigroup';
import { tuple } from './tuple';
// -------------------------------------------------------------------------------------
// internal
// -------------------------------------------------------------------------------------
/**
 * @internal
 */
export const prependW = (head) => (tail) => [head, ...tail];
/**
 * @internal
 */
export const prepend = prependW;
/**
 * @internal
 */
export const appendW = (end) => (init) => concatW([end])(init);
/**
 * @internal
 */
export const append = appendW;
/**
 * @internal
 */
export const isNonEmpty = _.isNonEmpty;
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Builds a `ReadonlyNonEmptyArray` from an array returning `none` if `as` is an empty array.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromReadonlyArray = (as) => isNonEmpty(as) ? _.some(as) : _.none;
/**
 * Return a `ReadonlyNonEmptyArray` of length `n` with element `i` initialized with `f(i)`.
 *
 * **Note**. `n` is normalized to a natural number.
 *
 * @example
 * import { makeBy } from 'fp-ts/ReadonlyNonEmptyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const double = (n: number): number => n * 2
 * assert.deepStrictEqual(pipe(5, makeBy(double)), [0, 2, 4, 6, 8])
 *
 * @category constructors
 * @since 3.0.0
 */
export const makeBy = (f) => (n) => {
    const j = Math.max(0, Math.floor(n));
    const out = [f(0)];
    for (let i = 1; i < j; i++) {
        out.push(f(i));
    }
    return out;
};
/**
 * Create a `ReadonlyNonEmptyArray` containing a value repeated the specified number of times.
 *
 * **Note**. `n` is normalized to a natural number.
 *
 * @example
 * import { replicate } from 'fp-ts/ReadonlyNonEmptyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(3, replicate('a')), ['a', 'a', 'a'])
 *
 * @category constructors
 * @since 3.0.0
 */
export const replicate = (a) => makeBy(() => a);
/**
 * Create a `ReadonlyNonEmptyArray` containing a range of integers, including both endpoints.
 *
 * @example
 * import { range } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(range(1, 5), [1, 2, 3, 4, 5])
 *
 * @category constructors
 * @since 3.0.0
 */
export const range = (start, end) => start <= end ? makeBy((i) => start + i)(end - start + 1) : [start];
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Produces a couple of the first element of the array, and a new array of the remaining elements, if any.
 *
 * @example
 * import { unprepend } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(unprepend([1, 2, 3, 4]), [1, [2, 3, 4]])
 *
 * @category destructors
 * @since 3.0.0
 */
export const unprepend = (as) => [head(as), tail(as)];
/**
 * Produces a couple of a copy of the array without its last element, and that last element.
 *
 * @example
 * import { unappend } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(unappend([1, 2, 3, 4]), [[1, 2, 3], 4])
 *
 * @category destructors
 * @since 3.0.0
 */
export const unappend = (as) => [init(as), last(as)];
/**
 * Break a `ReadonlyArray` into its first element and remaining elements.
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchLeft = (f) => (as) => f(head(as), tail(as));
/**
 * Break a `ReadonlyArray` into its initial elements and the last element.
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchRight = (f) => (as) => f(init(as), last(as));
export function concatW(second) {
    return (first) => first.concat(second);
}
/**
 * @category combinators
 * @since 3.0.0
 */
export const concat = concatW;
/**
 * Remove duplicates from a `ReadonlyNonEmptyArray`, keeping the first occurrence of an element.
 *
 * @example
 * import { uniq } from 'fp-ts/ReadonlyNonEmptyArray'
 * import * as N from 'fp-ts/number'
 *
 * assert.deepStrictEqual(uniq(N.Eq)([1, 2, 1]), [1, 2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const uniq = (E) => (as) => {
    if (as.length === 1) {
        return as;
    }
    const out = [head(as)];
    const rest = tail(as);
    for (const a of rest) {
        if (out.every((o) => !E.equals(o)(a))) {
            out.push(a);
        }
    }
    return out;
};
/**
 * Sort the elements of a `ReadonlyNonEmptyArray` in increasing order, where elements are compared using first `ords[0]`, then `ords[1]`,
 * etc...
 *
 * @example
 * import * as RNEA from 'fp-ts/ReadonlyNonEmptyArray'
 * import { contramap } from 'fp-ts/Ord'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * interface Person {
 *   name: string
 *   age: number
 * }
 *
 * const byName = pipe(S.Ord, contramap((p: Person) => p.name))
 *
 * const byAge = pipe(N.Ord, contramap((p: Person) => p.age))
 *
 * const sortByNameByAge = RNEA.sortBy([byName, byAge])
 *
 * const persons: RNEA.ReadonlyNonEmptyArray<Person> = [
 *   { name: 'a', age: 1 },
 *   { name: 'b', age: 3 },
 *   { name: 'c', age: 2 },
 *   { name: 'b', age: 2 }
 * ]
 *
 * assert.deepStrictEqual(sortByNameByAge(persons), [
 *   { name: 'a', age: 1 },
 *   { name: 'b', age: 2 },
 *   { name: 'b', age: 3 },
 *   { name: 'c', age: 2 }
 * ])
 *
 * @category combinators
 * @since 3.0.0
 */
export const sortBy = (ords) => {
    if (isNonEmpty(ords)) {
        const M = getMonoid();
        return sort(ords.reduce((a, acc) => M.concat(acc)(a), M.empty));
    }
    return identity;
};
/**
 * Creates a `ReadonlyArray` of unique values, in order, from all given `ReadonlyArray`s using a `Eq` for equality comparisons.
 *
 * @example
 * import { union } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2], union(N.Eq)([2, 3])), [1, 2, 3])
 *
 * @category combinators
 * @since 3.0.0
 */
export const union = (E) => {
    const uniqE = uniq(E);
    return (second) => (first) => uniqE(concat(second)(first));
};
/**
 * Rotate a `ReadonlyNonEmptyArray` by `n` steps.
 *
 * @example
 * import { rotate } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(rotate(2)([1, 2, 3, 4, 5]), [4, 5, 1, 2, 3])
 * assert.deepStrictEqual(rotate(-2)([1, 2, 3, 4, 5]), [3, 4, 5, 1, 2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const rotate = (n) => (as) => {
    const len = as.length;
    const m = Math.round(n) % len;
    if (isOutOfBound(Math.abs(m), as) || m === 0) {
        return as;
    }
    if (m < 0) {
        const [f, s] = splitAt(-m)(as);
        return concat(f)(s);
    }
    else {
        return rotate(m - len)(as);
    }
};
/**
 * Apply a function to the head, creating a new `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const modifyHead = (f) => (as) => [
    f(head(as)),
    ...tail(as)
];
/**
 * Change the head, creating a new `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const updateHead = (a) => modifyHead(() => a);
/**
 * Apply a function to the last element, creating a new `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const modifyLast = (f) => (as) => pipe(init(as), append(f(last(as))));
/**
 * Change the last element, creating a new `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const updateLast = (a) => modifyLast(() => a);
/**
 * Places an element in between members of a `ReadonlyNonEmptyArray`, then folds the results using the provided `Semigroup`.
 *
 * @example
 * import * as S from 'fp-ts/string'
 * import { intercalate } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(intercalate(S.Semigroup)('-')(['a', 'b', 'c']), 'a-b-c')
 *
 * @since 3.0.0
 */
export const intercalate = (S) => {
    const concatAllS = concatAll(S);
    return (middle) => flow(intersperse(middle), concatAllS);
};
export function comprehension(input, f) {
    const go = (as, input) => isNonEmpty(input)
        ? pipe(head(input), chain((head) => go(append(head)(as), tail(input))))
        : [f(...as)];
    return go(_.emptyReadonlyArray, input);
}
/**
 * @category combinators
 * @since 3.0.0
 */
export const reverse = (as) => as.length === 1 ? as : [last(as), ...as.slice(0, -1).reverse()];
/**
 * Group equal, consecutive elements of an array into non empty arrays.
 *
 * @example
 * import { group } from 'fp-ts/ReadonlyNonEmptyArray'
 * import * as N from 'fp-ts/number'
 *
 * assert.deepStrictEqual(group(N.Eq)([1, 2, 1, 1]), [
 *   [1],
 *   [2],
 *   [1, 1]
 * ])
 *
 * @category combinators
 * @since 3.0.0
 */
export const group = (E) => (as) => pipe(as, chop((as) => {
    const h = head(as);
    const out = [h];
    let i = 1;
    for (; i < as.length; i++) {
        const a = as[i];
        if (E.equals(h)(a)) {
            out.push(a);
        }
        else {
            break;
        }
    }
    return [out, as.slice(i)];
}));
/**
 * Splits an array into sub-non-empty-arrays stored in an object, based on the result of calling a `string`-returning
 * function on each element, and grouping the results according to values returned
 *
 * @example
 * import { groupBy } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(groupBy((s: string) => String(s.length))(['foo', 'bar', 'foobar']), {
 *   '3': ['foo', 'bar'],
 *   '6': ['foobar']
 * })
 *
 * @category combinators
 * @since 3.0.0
 */
export const groupBy = (f) => (as) => {
    const out = {};
    for (const a of as) {
        const k = f(a);
        if (_.has.call(out, k)) {
            out[k].push(a);
        }
        else {
            out[k] = [a];
        }
    }
    return out;
};
/**
 * Sort the elements of a `ReadonlyNonEmptyArray` in increasing order, creating a new `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const sort = (O) => (as) => as.length === 1 ? as : as.slice().sort((first, second) => O.compare(second)(first));
/**
 * @internal
 */
export const isOutOfBound = (i, as) => i < 0 || i >= as.length;
/**
 * Change the element at the specified index, creating a new `ReadonlyNonEmptyArray`, or returning `None` if the index is out of bounds.
 *
 * @since 3.0.0
 */
export const updateAt = (i, a) => modifyAt(i, () => a);
/**
 * Apply a function to the element at the specified index, creating a new `ReadonlyNonEmptyArray`, or returning `None` if the index is out
 * of bounds.
 *
 * @since 3.0.0
 */
export const modifyAt = (i, f) => (as) => {
    if (isOutOfBound(i, as)) {
        return _.none;
    }
    const prev = as[i];
    const next = f(prev);
    if (next === prev) {
        return _.some(as);
    }
    const out = fromReadonlyNonEmptyArray(as);
    out[i] = next;
    return _.some(out);
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const zipWith = (bs, f) => (as) => {
    const cs = [f(head(as), head(bs))];
    const len = Math.min(as.length, bs.length);
    for (let i = 1; i < len; i++) {
        cs[i] = f(as[i], bs[i]);
    }
    return cs;
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const zip = (bs) => (as) => pipe(as, zipWith(bs, tuple));
/**
 * @since 3.0.0
 */
export const unzip = (abs) => {
    const fa = [abs[0][0]];
    const fb = [abs[0][1]];
    for (let i = 1; i < abs.length; i++) {
        fa[i] = abs[i][0];
        fb[i] = abs[i][1];
    }
    return [fa, fb];
};
/**
 * Prepend an element to every member of an array
 *
 * @example
 * import { prependAll } from 'fp-ts/ReadonlyNonEmptyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3, 4], prependAll(9)), [9, 1, 9, 2, 9, 3, 9, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const prependAll = (middle) => (as) => {
    const out = [middle, head(as)];
    for (let i = 1; i < as.length; i++) {
        out.push(middle, as[i]);
    }
    return out;
};
/**
 * Places an element in between members of an array
 *
 * @example
 * import { intersperse } from 'fp-ts/ReadonlyNonEmptyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3, 4], intersperse(9)), [1, 9, 2, 9, 3, 9, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const intersperse = (middle) => (as) => {
    const rest = tail(as);
    return isNonEmpty(rest) ? prepend(head(as))(prependAll(middle)(rest)) : as;
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainWithIndex = (f) => (as) => {
    const out = fromReadonlyNonEmptyArray(f(0, head(as)));
    for (let i = 1; i < as.length; i++) {
        out.push(...f(i, as[i]));
    }
    return out;
};
/**
 * A useful recursion pattern for processing a `ReadonlyNonEmptyArray` to produce a new `ReadonlyNonEmptyArray`, often used for "chopping" up the input
 * `ReadonlyNonEmptyArray`. Typically `chop` is called with some function that will consume an initial prefix of the `ReadonlyNonEmptyArray` and produce a
 * value and the tail of the `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const chop = (f) => (as) => {
    const [b, rest] = f(as);
    const out = [b];
    let next = rest;
    while (isNonEmpty(next)) {
        const [b, rest] = f(next);
        out.push(b);
        next = rest;
    }
    return out;
};
/**
 * Splits a `ReadonlyNonEmptyArray` into two pieces, the first piece has max `n` elements.
 *
 * @category combinators
 * @since 3.0.0
 */
export const splitAt = (n) => (as) => {
    const m = Math.max(1, n);
    return m >= as.length ? [as, _.emptyReadonlyArray] : [pipe(as.slice(1, m), prepend(head(as))), as.slice(m)];
};
/**
 * Splits a `ReadonlyNonEmptyArray` into length-`n` pieces. The last piece will be shorter if `n` does not evenly divide the length of
 * the `ReadonlyNonEmptyArray`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const chunksOf = (n) => chop(splitAt(n));
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export const altW = (second) => concatW(second());
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export const alt = altW;
/**
 * @category Apply
 * @since 3.0.0
 */
export const ap = (fa) => chain((f) => pipe(fa, map(f)));
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = _.singleton;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = (f) => chainWithIndex((_, a) => f(a));
/**
 * @category Extend
 * @since 3.0.0
 */
export const extend = (f) => (as) => {
    let next = tail(as);
    const out = [f(as)];
    while (isNonEmpty(next)) {
        out.push(f(next));
        next = tail(next);
    }
    return out;
};
/**
 * Derivable from `Extend`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const duplicate = 
/*#__PURE__*/
extend(identity);
/**
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = 
/*#__PURE__*/
chain(identity);
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => mapWithIndex((_, a) => f(a));
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export const mapWithIndex = (f) => (as) => {
    const out = [f(0, head(as))];
    for (let i = 1; i < as.length; i++) {
        out.push(f(i, as[i]));
    }
    return out;
};
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => reduceWithIndex(b, (_, b, a) => f(b, a));
/**
 * **Note**. The constraint is relaxed: a `Semigroup` instead of a `Monoid`.
 *
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = (S) => (f) => (fa) => fa.slice(1).reduce((s, a) => S.concat(f(a))(s), f(fa[0]));
/**
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const reduceWithIndex = (b, f) => (as) => as.reduce((b, a, i) => f(i, b, a), b);
/**
 * **Note**. The constraint is relaxed: a `Semigroup` instead of a `Monoid`.
 *
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const foldMapWithIndex = (S) => (f) => (fa) => fa.slice(1).reduce((s, a, i) => S.concat(f(i + 1, a))(s), f(0, fa[0]));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => reduceRightWithIndex(b, (_, b, a) => f(b, a));
/**
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const reduceRightWithIndex = (b, f) => (as) => as.reduceRight((b, a, i) => f(i, a, b), b);
/**
 * @since 3.0.0
 */
export const traverse = (F) => {
    const traverseWithIndexF = traverseWithIndex(F);
    return (f) => traverseWithIndexF((_, a) => f(a));
};
/**
 * @since 3.0.0
 */
export const traverseWithIndex = (F) => (f) => (as) => {
    let out = pipe(f(0, head(as)), F.map(of));
    for (let i = 1; i < as.length; i++) {
        out = pipe(out, F.map((bs) => (b) => pipe(bs, append(b))), F.ap(f(i, as[i])));
    }
    return out;
};
/**
 * @since 3.0.0
 */
export const extract = _.head;
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => ({
    show: (as) => `[${as.map(S.show).join(', ')}]`
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = () => ({
    concat
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (E) => fromEquals((second) => (first) => first.length === second.length && first.every((a, i) => E.equals(second[i])(a)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const getUnionSemigroup = (E) => ({
    concat: union(E)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FunctorWithIndex = {
    mapWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FoldableWithIndex = {
    reduceWithIndex,
    foldMapWithIndex,
    reduceRightWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    map,
    traverse
};
/**
 * @category instances
 * @since 3.0.0
 */
export const TraversableWithIndex = {
    traverseWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Alt = {
    map,
    alt
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = 
/*#__PURE__*/
of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/
tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/
apT_(Apply);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const head = extract;
/**
 * @since 3.0.0
 */
export const tail = _.tail;
/**
 * @since 3.0.0
 */
export const last = (as) => as[as.length - 1];
/**
 * Get all but the last element of a non empty array, creating a new array.
 *
 * @example
 * import { init } from 'fp-ts/ReadonlyNonEmptyArray'
 *
 * assert.deepStrictEqual(init([1, 2, 3]), [1, 2])
 * assert.deepStrictEqual(init([1]), [])
 *
 * @since 3.0.0
 */
export const init = (as) => as.slice(0, -1);
/**
 * @since 3.0.0
 */
export const min = (O) => {
    const S = Se.min(O);
    return (nea) => nea.reduce((a, acc) => S.concat(acc)(a));
};
/**
 * @since 3.0.0
 */
export const max = (O) => {
    const S = Se.max(O);
    return (nea) => nea.reduce((a, acc) => S.concat(acc)(a));
};
/**
 * @since 3.0.0
 */
export const concatAll = (S) => (fa) => fa.reduce((a, acc) => S.concat(acc)(a));
