import { fromEquals } from './Eq';
import { pipe } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
import * as O from './Option';
import { separated } from './Separated';
import { wiltDefault, witherDefault } from './Witherable';
import * as S from './string';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Create a `ReadonlyRecord` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export const singleton = (k, a) => ({ [k]: a });
export function fromFoldable(F) {
    return (M) => (f) => F.reduce({}, (r, a) => {
        const [k, b] = f(a);
        r[k] = _.has.call(r, k) ? M.concat(b)(r[k]) : b;
        return r;
    });
}
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Insert an element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export const insertAt = (k, a) => (r) => {
    if (!_.has.call(r, k)) {
        const out = Object.assign({}, r);
        out[k] = a;
        return _.some(out);
    }
    return _.none;
};
/**
 * Insert or replace a key/value pair in a `ReadonlyRecord`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const upsertAt = (k, a) => (r) => {
    if (_.has.call(r, k) && r[k] === a) {
        return r;
    }
    const out = Object.assign({}, r);
    out[k] = a;
    return out;
};
/**
 * Change the element at the specified keys, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const updateAt = (k, a) => modifyAt(k, () => a);
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const modifyAt = (k, f) => (r) => {
    if (!has(k, r)) {
        return _.none;
    }
    const a = f(r[k]);
    if (a === r[k]) {
        return _.some(r);
    }
    const out = Object.assign({}, r);
    out[k] = a;
    return _.some(out);
};
/**
 * Delete the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const deleteAt = (k) => (r) => {
    if (!_.has.call(r, k)) {
        return _.none;
    }
    const out = Object.assign({}, r);
    delete out[k];
    return _.some(out);
};
/**
 * Delete the element at the specified key, returning the value as well as the subsequent `ReadonlyRecord`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export const pop = (k) => (r) => pipe(r, deleteAt(k), O.map((out) => [r[k], out]));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a `ReadonlyRecord` passing the values to the iterating function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => mapWithIndex((_, a) => f(a));
export function mapWithIndex(f) {
    return (r) => {
        const out = {};
        for (const k in r) {
            if (_.has.call(r, k)) {
                out[k] = f(k, r[k]);
            }
        }
        return out;
    };
}
/**
 * @since 3.0.0
 */
export const keys = (O) => (r) => Object.keys(r).sort((first, second) => O.compare(second)(first));
/**
 * @since 3.0.0
 */
export const reduceWithIndex = (O) => {
    const keysO = keys(O);
    return (b, f) => (r) => {
        let out = b;
        for (const k of keysO(r)) {
            out = f(k, out, r[k]);
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const foldMapWithIndex = (O) => {
    const keysO = keys(O);
    return (M) => (f) => (r) => {
        let out = M.empty;
        for (const k of keysO(r)) {
            out = M.concat(f(k, r[k]))(out);
        }
        return out;
    };
};
/**
 * @since 3.0.0
 */
export const reduceRightWithIndex = (O) => {
    const keysO = keys(O);
    return (b, f) => (r) => {
        let out = b;
        const ks = keysO(r);
        const len = ks.length;
        for (let i = len - 1; i >= 0; i--) {
            const k = ks[i];
            out = f(k, r[k], out);
        }
        return out;
    };
};
export function traverseWithIndex(O) {
    const keysO = keys(O);
    return (F) => (f) => (ta) => {
        if (isEmpty(ta)) {
            return F.of(empty);
        }
        let out = F.of({});
        for (const key of keysO(ta)) {
            out = pipe(out, F.map((r) => (b) => Object.assign({}, r, { [key]: b })), F.ap(f(key, ta[key])));
        }
        return out;
    };
}
export function traverse(O) {
    const traverseWithIndexO = traverseWithIndex(O);
    return (F) => {
        const traverseWithIndexOF = traverseWithIndexO(F);
        return (f) => traverseWithIndexOF((_, a) => f(a));
    };
}
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export function partitionMapWithIndex(f) {
    return (r) => {
        const left = {};
        const right = {};
        for (const k in r) {
            if (_.has.call(r, k)) {
                const e = f(k, r[k]);
                if (_.isLeft(e)) {
                    left[k] = e.left;
                }
                else {
                    right[k] = e.right;
                }
            }
        }
        return separated(left, right);
    };
}
export function partitionWithIndex(predicateWithIndex) {
    return (r) => {
        const left = {};
        const right = {};
        for (const k in r) {
            if (_.has.call(r, k)) {
                const a = r[k];
                if (predicateWithIndex(k, a)) {
                    right[k] = a;
                }
                else {
                    left[k] = a;
                }
            }
        }
        return separated(left, right);
    };
}
export function filterMapWithIndex(f) {
    return (r) => {
        const out = {};
        for (const k in r) {
            if (_.has.call(r, k)) {
                const ob = f(k, r[k]);
                if (_.isSome(ob)) {
                    out[k] = ob.value;
                }
            }
        }
        return out;
    };
}
export function filterWithIndex(predicateWithIndex) {
    return (r) => {
        const out = {};
        let changed = false;
        for (const key in r) {
            if (_.has.call(r, key)) {
                const a = r[key];
                if (predicateWithIndex(key, a)) {
                    out[key] = a;
                }
                else {
                    changed = true;
                }
            }
        }
        return changed ? out : r;
    };
}
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filter = (predicate) => filterWithIndex((_, a) => predicate(a));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = (f) => filterMapWithIndex((_, a) => f(a));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partition = (predicate) => partitionWithIndex((_, a) => predicate(a));
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = (f) => partitionMapWithIndex((_, a) => f(a));
/**
 * @since 3.0.0
 */
export const reduce = (O) => {
    const reduceWithIndexO = reduceWithIndex(O);
    return (b, f) => reduceWithIndexO(b, (_, b, a) => f(b, a));
};
/**
 * @since 3.0.0
 */
export const foldMap = (O) => {
    const foldMapWithIndexO = foldMapWithIndex(O);
    return (M) => {
        const foldMapWithIndexOM = foldMapWithIndexO(M);
        return (f) => foldMapWithIndexOM((_, a) => f(a));
    };
};
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (O) => {
    const reduceRightWithIndexO = reduceRightWithIndex(O);
    return (b, f) => reduceRightWithIndexO(b, (_, b, a) => f(b, a));
};
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = (r) => {
    const out = {};
    for (const k in r) {
        if (_.has.call(r, k)) {
            const oa = r[k];
            if (_.isSome(oa)) {
                out[k] = oa.value;
            }
        }
    }
    return out;
};
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = (r) => {
    const left = {};
    const right = {};
    for (const k in r) {
        if (_.has.call(r, k)) {
            const e = r[k];
            if (_.isLeft(e)) {
                left[k] = e.left;
            }
            else {
                right[k] = e.right;
            }
        }
    }
    return separated(left, right);
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (O) => {
    const collectO = collect(O);
    return (S) => {
        const f = collectO((k, a) => `${JSON.stringify(k)}: ${S.show(a)}`);
        return {
            show: (r) => {
                const elements = f(r).join(', ');
                return elements === '' ? '{}' : `{ ${elements} }`;
            }
        };
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export function getEq(E) {
    const isSubrecordE = isSubrecord(E);
    return fromEquals((second) => (first) => isSubrecordE(first)(second) && isSubrecordE(second)(first));
}
export function getMonoid(S) {
    return {
        concat: (second) => (first) => {
            if (isEmpty(first)) {
                return second;
            }
            if (isEmpty(second)) {
                return first;
            }
            const r = Object.assign({}, first);
            for (const k in second) {
                if (_.has.call(second, k)) {
                    r[k] = _.has.call(first, k) ? S.concat(second[k])(first[k]) : second[k];
                }
            }
            return r;
        },
        empty
    };
}
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const FunctorWithIndex = {
    mapWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFoldable = (O) => ({
    reduce: reduce(O),
    foldMap: foldMap(O),
    reduceRight: reduceRight(O)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getFoldableWithIndex = (O) => ({
    reduceWithIndex: reduceWithIndex(O),
    foldMapWithIndex: foldMapWithIndex(O),
    reduceRightWithIndex: reduceRightWithIndex(O)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    filter,
    filterMap,
    partition,
    partitionMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FilterableWithIndex = {
    filterMapWithIndex,
    filterWithIndex,
    partitionMapWithIndex,
    partitionWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getTraversable = (O) => ({
    map,
    traverse: traverse(O)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getTraversableWithIndex = (O) => ({
    traverseWithIndex: traverseWithIndex(O)
});
/**
 * @category Witherable
 * @since 3.0.0
 */
export const wither = (O) => witherDefault(getTraversable(O), Compactable);
/**
 * @category Witherable
 * @since 3.0.0
 */
export const wilt = (O) => wiltDefault(getTraversable(O), Compactable);
/**
 * @category instances
 * @since 3.0.0
 */
export const getWitherable = (O) => ({
    wither: wither(O),
    wilt: wilt(O)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionSemigroup = (S) => ({
    concat: union(S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionMonoid = (S) => ({
    concat: getUnionSemigroup(S).concat,
    empty
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getIntersectionSemigroup = (S) => ({
    concat: intersection(S)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getDifferenceMagma = () => ({
    concat: difference
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Calculate the number of key/value pairs in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const size = (r) => Object.keys(r).length;
/**
 * Test whether a `ReadonlyRecord` is empty.
 *
 * @since 3.0.0
 */
export const isEmpty = (r) => {
    for (const k in r) {
        if (_.has.call(r, k)) {
            return false;
        }
    }
    return true;
};
/**
 * Map a `ReadonlyRecord` into an `ReadonlyArray`.
 *
 * @example
 * import { collect } from 'fp-ts/ReadonlyRecord'
 * import * as S from 'fp-ts/string'
 *
 * const x: { a: string, b: boolean } = { a: 'foo', b: false }
 * assert.deepStrictEqual(
 *   collect(S.Ord)((key, val) => ({key: key, value: val}))(x),
 *   [{key: 'a', value: 'foo'}, {key: 'b', value: false}]
 * )
 *
 * @since 3.0.0
 */
export const collect = (O) => {
    const keysO = keys(O);
    return (f) => (r) => {
        const out = [];
        for (const k of keysO(r)) {
            out.push(f(k, r[k]));
        }
        return out;
    };
};
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const toReadonlyArray = (O) => collect(O)((k, a) => [k, a]);
export function toUnfoldable(O) {
    const toReadonlyArrayO = toReadonlyArray(O);
    return (U) => (r) => {
        const as = toReadonlyArrayO(r);
        const len = as.length;
        return U.unfold(0, (b) => (b < len ? _.some([as[b], b + 1]) : _.none));
    };
}
/**
 * Test whether or not a key exists in a `ReadonlyRecord`.
 *
 * Note. This function is not pipeable because is a custom type guard.
 *
 * @since 3.0.0
 */
export const has = (k, r) => _.has.call(r, k);
/**
 * Test whether one `ReadonlyRecord` contains all of the keys and values contained in another `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const isSubrecord = (E) => (second) => (first) => {
    for (const k in first) {
        if (!_.has.call(second, k) || !E.equals(second[k])(first[k])) {
            return false;
        }
    }
    return true;
};
/**
 * Lookup the value for a key in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const lookup = (k) => (r) => _.has.call(r, k) ? _.some(r[k]) : _.none;
/**
 * An empty `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const empty = _.emptyRecord;
export function every(predicate) {
    return (r) => {
        for (const k in r) {
            if (!predicate(r[k])) {
                return false;
            }
        }
        return true;
    };
}
/**
 * @since 3.0.0
 */
export const some = (predicate) => (r) => {
    for (const k in r) {
        if (predicate(r[k])) {
            return true;
        }
    }
    return false;
};
/**
 * Tests whether a value is a member of a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => {
    const predicate = E.equals(a);
    return (r) => {
        for (const k in r) {
            if (predicate(r[k])) {
                return true;
            }
        }
        return false;
    };
};
/**
 * @since 3.0.0
 */
export const union = (M) => (second) => (first) => {
    if (isEmpty(first)) {
        return second;
    }
    if (isEmpty(second)) {
        return first;
    }
    const out = {};
    for (const k in first) {
        if (has(k, second)) {
            out[k] = M.concat(second[k])(first[k]);
        }
        else {
            out[k] = first[k];
        }
    }
    for (const k in second) {
        if (!has(k, out)) {
            out[k] = second[k];
        }
    }
    return out;
};
/**
 * @since 3.0.0
 */
export const intersection = (M) => (second) => (first) => {
    if (isEmpty(first) || isEmpty(second)) {
        return empty;
    }
    const out = {};
    for (const k in first) {
        if (has(k, second)) {
            out[k] = M.concat(second[k])(first[k]);
        }
    }
    return out;
};
/**
 * @since 3.0.0
 */
export const difference = (second) => (first) => {
    if (isEmpty(first)) {
        return second;
    }
    if (isEmpty(second)) {
        return first;
    }
    const out = {};
    for (const k in first) {
        if (!has(k, second)) {
            out[k] = first[k];
        }
    }
    for (const k in second) {
        if (!has(k, first)) {
            out[k] = second[k];
        }
    }
    return out;
};
/**
 * Converts a `ReadonlyRecord` into a `ReadonlyArray` of `[key, value]` tuples.
 *
 * @example
 * import { toEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(toEntries({ a: 1, b: 2 }), [['a', 1], ['b', 2]])
 *
 * @since 3.0.0
 */
export const toEntries = toReadonlyArray(S.Ord);
/**
 * Converts a `ReadonlyArray` of `[key, value]` tuples into a `ReadonlyRecord`.
 *
 * @example
 * import { fromEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(fromEntries([['a', 1], ['b', 2], ['a', 3]]), { b: 2, a: 3 })
 *
 * @since 3.0.0
 */
export const fromEntries = (fa) => {
    const out = {};
    for (const a of fa) {
        out[a[0]] = a[1];
    }
    return out;
};
