// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Given a tuple of `Ring`s returns a `Ring` for the tuple
 *
 * @example
 * import { tuple } from 'fp-ts/Ring'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const R = tuple(N.Field, N.Field, N.Field)
 * assert.deepStrictEqual(pipe([1, 2, 3], R.add([4, 5, 6])), [5, 7, 9])
 * assert.deepStrictEqual(pipe([1, 2, 3], R.mul([4, 5, 6])), [4, 10, 18])
 * assert.deepStrictEqual(R.one, [1, 1, 1])
 * assert.deepStrictEqual(pipe([1, 2, 3], R.sub([4, 5, 6])), [-3, -3, -3])
 * assert.deepStrictEqual(R.zero, [0, 0, 0])
 *
 * @category combinators
 * @since 3.0.0
 */
export const tuple = (...rings) => ({
    add: (second) => (first) => rings.map((R, i) => R.add(second[i])(first[i])),
    zero: rings.map((R) => R.zero),
    mul: (second) => (first) => rings.map((R, i) => R.mul(second[i])(first[i])),
    one: rings.map((R) => R.one),
    sub: (second) => (first) => rings.map((R, i) => R.sub(second[i])(first[i]))
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * `negate x` can be used as a shorthand for `zero - x`
 *
 * @since 3.0.0
 */
export const negate = (ring) => (a) => ring.sub(a)(ring.zero);
