import { map as map_, mapLeftDefault } from './Bifunctor';
import { fromEquals } from './Eq';
import { fromOption as fromOption_, fromOptionK as fromOptionK_, fromPredicate as fromPredicate_ } from './FromEither';
import { identity, pipe } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = (left) => ({ _tag: 'Left', left });
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = (right) => ({ _tag: 'Right', right });
/**
 * @category constructors
 * @since 3.0.0
 */
export const both = (left, right) => ({ _tag: 'Both', left, right });
/**
 * @example
 * import { leftOrBoth, left, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(leftOrBoth(() => 'a')(none), left('a'))
 * assert.deepStrictEqual(leftOrBoth(() => 'a')(some(1)), both('a', 1))
 *
 * @category constructors
 * @since 3.0.0
 */
export const leftOrBoth = (e) => (ma) => _.isNone(ma) ? left(e()) : both(e(), ma.value);
/**
 * @example
 * import { rightOrBoth, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(rightOrBoth(() => 1)(none), right(1))
 * assert.deepStrictEqual(rightOrBoth(() => 1)(some('a')), both('a', 1))
 *
 * @category constructors
 * @since 3.0.0
 */
export const rightOrBoth = (a) => (me) => _.isNone(me) ? right(a()) : both(me.value, a());
/**
 * Takes a pair of `Option`s and attempts to create a `These` from them
 *
 * @example
 * import { fromOptions, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(fromOptions(none, none), none)
 * assert.deepStrictEqual(fromOptions(some('a'), none), some(left('a')))
 * assert.deepStrictEqual(fromOptions(none, some(1)), some(right(1)))
 * assert.deepStrictEqual(fromOptions(some('a'), some(1)), some(both('a', 1)))
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromOptions = (fe, fa) => _.isNone(fe)
    ? _.isNone(fa)
        ? _.none
        : _.some(right(fa.value))
    : _.isNone(fa)
        ? _.some(left(fe.value))
        : _.some(both(fe.value, fa.value));
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchW = (onLeft, onRight, onBoth) => (fa) => {
    switch (fa._tag) {
        case 'Left':
            return onLeft(fa.left);
        case 'Right':
            return onRight(fa.right);
        case 'Both':
            return onBoth(fa.left, fa.right);
    }
};
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = matchW;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = match(right, left, (e, a) => both(a, e));
// -------------------------------------------------------------------------------------
// refinements
// -------------------------------------------------------------------------------------
/**
 * Returns `true` if the these is an instance of `Left`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
export const isLeft = (fa) => fa._tag === 'Left';
/**
 * Returns `true` if the these is an instance of `Right`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
export const isRight = (fa) => fa._tag === 'Right';
/**
 * Returns `true` if the these is an instance of `Both`, `false` otherwise
 *
 * @category guards
 * @since 3.0.0
 */
export const isBoth = (fa) => fa._tag === 'Both';
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = (f, g) => (fa) => isLeft(fa) ? left(f(fa.left)) : isRight(fa) ? right(g(fa.right)) : both(f(fa.left), g(fa.right));
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = 
/*#__PURE__*/
mapLeftDefault(bimap);
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => (isLeft(fa) ? b : f(b, fa.right));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = (M) => (f) => (fa) => (isLeft(fa) ? M.empty : f(fa.right));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => (isLeft(fa) ? b : f(fa.right, b));
/**
 * @since 3.0.0
 */
export const traverse = (F) => (f) => (ta) => isLeft(ta)
    ? F.of(ta)
    : isRight(ta)
        ? pipe(f(ta.right), F.map(right))
        : pipe(f(ta.right), F.map((b) => both(ta.left, b)));
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (SE, SA) => ({
    show: match((l) => `left(${SE.show(l)})`, (a) => `right(${SA.show(a)})`, (l, a) => `both(${SE.show(l)}, ${SA.show(a)})`)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (EE, EA) => fromEquals((second) => (first) => isLeft(first)
    ? isLeft(second) && EE.equals(second.left)(first.left)
    : isRight(first)
        ? isRight(second) && EA.equals(second.right)(first.right)
        : isBoth(second) && EE.equals(second.left)(first.left) && EA.equals(second.right)(first.right));
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = (SE, SA) => ({
    concat: (second) => (first) => isLeft(first)
        ? isLeft(second)
            ? left(SE.concat(second.left)(first.left))
            : isRight(second)
                ? both(first.left, second.right)
                : both(SE.concat(second.left)(first.left), second.right)
        : isRight(first)
            ? isLeft(second)
                ? both(second.left, first.right)
                : isRight(second)
                    ? right(SA.concat(second.right)(first.right))
                    : both(second.left, SA.concat(second.right)(first.right))
            : isLeft(second)
                ? both(SE.concat(second.left)(first.left), first.right)
                : isRight(second)
                    ? both(first.left, SA.concat(second.right)(first.right))
                    : both(SE.concat(second.left)(first.left), SA.concat(second.right)(first.right))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft
};
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = 
/*#__PURE__*/
map_(Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (fab) => isLeft(fab)
        ? isLeft(fa)
            ? left(S.concat(fa.left)(fab.left))
            : isRight(fa)
                ? left(fab.left)
                : left(S.concat(fa.left)(fab.left))
        : isRight(fab)
            ? isLeft(fa)
                ? left(fa.left)
                : isRight(fa)
                    ? right(fab.right(fa.right))
                    : both(fa.left, fab.right(fa.right))
            : isLeft(fa)
                ? left(S.concat(fa.left)(fab.left))
                : isRight(fa)
                    ? both(fab.left, fab.right(fa.right))
                    : both(S.concat(fa.left)(fab.left), fab.right(fa.right))
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (S) => {
    const A = getApply(S);
    return {
        map,
        ap: A.ap,
        of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => {
    const chain = (f) => (ma) => {
        if (isLeft(ma)) {
            return ma;
        }
        if (isRight(ma)) {
            return f(ma.right);
        }
        const fb = f(ma.right);
        return isLeft(fb)
            ? left(S.concat(fb.left)(ma.left))
            : isRight(fb)
                ? both(ma.left, fb.right)
                : both(S.concat(fb.left)(ma.left), fb.right);
    };
    return {
        map,
        chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (S) => {
    const C = getChain(S);
    return {
        map,
        of,
        chain: C.chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither: identity
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/
fromOption_(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = 
/*#__PURE__*/
fromOptionK_(FromEither);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/
fromPredicate_(FromEither);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromThese = {
    fromThese: identity
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    map,
    traverse
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const elem = (E) => (a) => exists(E.equals(a));
/**
 * @since 3.0.0
 */
export const exists = (predicate) => (ma) => isLeft(ma) ? false : predicate(ma.right);
/**
 * @example
 * import { toTuple2, left, right, both } from 'fp-ts/These'
 *
 * const f = toTuple2(() => 'a', () => 1)
 * assert.deepStrictEqual(f(left('b')), ['b', 1])
 * assert.deepStrictEqual(f(right(2)), ['a', 2])
 * assert.deepStrictEqual(f(both('b', 2)), ['b', 2])
 *
 * @since 3.0.0
 */
export const toTuple2 = (e, a) => (fa) => isLeft(fa) ? [fa.left, a()] : isRight(fa) ? [e(), fa.right] : [fa.left, fa.right];
/**
 * Returns an `E` value if possible
 *
 * @example
 * import { getLeft, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getLeft(left('a')), some('a'))
 * assert.deepStrictEqual(getLeft(right(1)), none)
 * assert.deepStrictEqual(getLeft(both('a', 1)), some('a'))
 *
 * @since 3.0.0
 */
export const getLeft = (fa) => isLeft(fa) ? _.some(fa.left) : isRight(fa) ? _.none : _.some(fa.left);
/**
 * Returns an `A` value if possible
 *
 * @example
 * import { getRight, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getRight(left('a')), none)
 * assert.deepStrictEqual(getRight(right(1)), some(1))
 * assert.deepStrictEqual(getRight(both('a', 1)), some(1))
 *
 * @since 3.0.0
 */
export const getRight = (fa) => isLeft(fa) ? _.none : isRight(fa) ? _.some(fa.right) : _.some(fa.right);
/**
 * Returns the `E` value if and only if the value is constructed with `Left`
 *
 * @example
 * import { getLeftOnly, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getLeftOnly(left('a')), some('a'))
 * assert.deepStrictEqual(getLeftOnly(right(1)), none)
 * assert.deepStrictEqual(getLeftOnly(both('a', 1)), none)
 *
 * @since 3.0.0
 */
export const getLeftOnly = (fa) => (isLeft(fa) ? _.some(fa.left) : _.none);
/**
 * Returns the `A` value if and only if the value is constructed with `Right`
 *
 * @example
 * import { getRightOnly, left, right, both } from 'fp-ts/These'
 * import { none, some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(getRightOnly(left('a')), none)
 * assert.deepStrictEqual(getRightOnly(right(1)), some(1))
 * assert.deepStrictEqual(getRightOnly(both('a', 1)), none)
 *
 * @since 3.0.0
 */
export const getRightOnly = (fa) => (isRight(fa) ? _.some(fa.right) : _.none);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApplicative(S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (S) => (f) => (as) => {
    let e = _.none;
    const t = f(0, _.head(as));
    if (isLeft(t)) {
        return t;
    }
    if (isBoth(t)) {
        e = _.some(t.left);
    }
    const out = [t.right];
    for (let i = 1; i < as.length; i++) {
        const t = f(i, as[i]);
        if (isLeft(t)) {
            return t;
        }
        if (isBoth(t)) {
            e = _.isNone(e) ? _.some(t.left) : _.some(S.concat(t.left)(e.value));
        }
        out.push(t.right);
    }
    return _.isNone(e) ? right(out) : both(e.value, out);
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (S) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(S)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
