import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { bind as bind_, chainFirst as chainFirst_ } from './Chain';
import { fromEquals } from './Eq';
import { flow, identity, pipe } from './function';
import { bindTo as bindTo_, flap as flap_, tupled as tupled_ } from './Functor';
import * as _ from './internal';
import * as RA from './ReadonlyArray';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const tree = (value, forest = RA.empty) => ({
    value,
    forest
});
/**
 * Build a (possibly infinite) tree from a seed value in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldTree = (f) => (b) => {
    const [a, bs] = f(b);
    return {
        value: a,
        forest: pipe(bs, unfoldForest(f))
    };
};
/**
 * Build a (possibly infinite) forest from a list of seed values in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldForest = (f) => (bs) => bs.map(unfoldTree(f));
export function unfoldTreeM(M) {
    const unfoldForestMM = unfoldForestM(M);
    return (f) => flow(f, M.chain(([value, bs]) => pipe(bs, unfoldForestMM(f), M.map((forest) => ({ value, forest })))));
}
export function unfoldForestM(M) {
    const traverseM = RA.traverse(M);
    return (f) => traverseM(unfoldTreeM(M)(f));
}
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Fold a tree into a "summary" value in depth-first order.
 *
 * For each node in the tree, apply `f` to the `value` and the result of applying `f` to each `forest`.
 *
 * This is also known as the catamorphism on trees.
 *
 * @example
 * import { fold, tree } from 'fp-ts/Tree'
 * import * as N from 'fp-ts/number'
 * import { concatAll } from 'fp-ts/Monoid'
 * import { pipe } from 'fp-ts/function'
 * import { isEmpty } from 'fp-ts/ReadonlyArray'
 *
 * const t = tree(1, [tree(2), tree(3)])
 *
 * const sum = concatAll(N.MonoidSum)
 *
 * assert.deepStrictEqual(pipe(t, fold((a, bs) => a + sum(bs))), 6)
 * assert.deepStrictEqual(pipe(t, fold((a, bs) => bs.reduce((b, acc) => Math.max(b, acc), a))), 3)
 * assert.deepStrictEqual(pipe(t, fold((_, bs) => (isEmpty(bs) ? 1 : sum(bs)))), 2)
 *
 * @category destructors
 * @since 3.0.0
 */
export const fold = (f) => {
    const go = (tree) => f(tree.value, tree.forest.map(go));
    return go;
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => ({
    value: f(fa.value),
    forest: fa.forest.map(map(f))
});
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = (fa) => chain((f) => pipe(fa, map(f)));
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = (f) => (ma) => {
    const { value, forest } = f(ma.value);
    const concat = RA.getMonoid().concat;
    return {
        value,
        forest: concat(ma.forest.map(chain(f)))(forest)
    };
};
/**
 * @category Extend
 * @since 3.0.0
 */
export const extend = (f) => (wa) => ({
    value: f(wa),
    forest: wa.forest.map(extend(f))
});
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const duplicate = 
/*#__PURE__*/
extend(identity);
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const flatten = 
/*#__PURE__*/
chain(identity);
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => {
    let r = f(b, fa.value);
    const len = fa.forest.length;
    for (let i = 0; i < len; i++) {
        r = pipe(fa.forest[i], reduce(r, f));
    }
    return r;
};
/**
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = (M) => (f) => reduce(M.empty, (acc, a) => M.concat(f(a))(acc));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => {
    let r = b;
    const len = fa.forest.length;
    for (let i = len - 1; i >= 0; i--) {
        r = pipe(fa.forest[i], reduceRight(r, f));
    }
    return f(fa.value, r);
};
/**
 * @category Extract
 * @since 3.0.0
 */
export const extract = (wa) => wa.value;
/**
 * @since 3.0.0
 */
export const traverse = (F) => {
    const traverseF = RA.traverse(F);
    const out = (f) => (ta) => pipe(f(ta.value), F.map((value) => (forest) => ({
        value,
        forest
    })), F.ap(pipe(ta.forest, traverseF(out(f)))));
    return out;
};
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = (a) => tree(a);
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => {
    const show = (t) => {
        return RA.isEmpty(t.forest)
            ? `tree(${S.show(t.value)})`
            : `tree(${S.show(t.value)}, [${t.forest.map(show).join(', ')}])`;
    };
    return {
        show
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (E) => {
    const R = fromEquals((second) => (first) => E.equals(second.value)(first.value) && SA.equals(second.forest)(first.forest));
    const SA = RA.getEq(R);
    return R;
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = 
/*#__PURE__*/
apFirst_(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = 
/*#__PURE__*/
apSecond_(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = 
/*#__PURE__*/
chainFirst_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    map,
    traverse
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const exists = (predicate) => (ma) => predicate(ma.value) || ma.forest.some(exists(predicate));
/**
 * Tests whether a value is a member of a `Tree`.
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => exists(E.equals(a));
const draw = (indentation, forest) => {
    let r = '';
    const len = forest.length;
    let tree;
    for (let i = 0; i < len; i++) {
        tree = forest[i];
        const isLast = i === len - 1;
        r += indentation + (isLast ? '└' : '├') + '─ ' + tree.value;
        r += draw(indentation + (len > 1 && !isLast ? '│  ' : '   '), tree.forest);
    }
    return r;
};
/**
 * Neat 2-dimensional drawing of a forest
 *
 * @since 3.0.0
 */
export const drawForest = (forest) => draw('\n', forest);
/**
 * Neat 2-dimensional drawing of a tree
 *
 * @example
 * import { tree, drawTree } from 'fp-ts/Tree'
 *
 * const fa = tree('a', [
 *   tree('b'),
 *   tree('c'),
 *   tree('d', [tree('e'), tree('f')])
 * ])
 *
 * assert.strictEqual(drawTree(fa), `a
 * ├─ b
 * ├─ c
 * └─ d
 *    ├─ e
 *    └─ f`)
 *
 *
 * @since 3.0.0
 */
export const drawTree = (tree) => tree.value + drawForest(tree.forest);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = 
/*#__PURE__*/
of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/
bindTo_(Functor);
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/
bind_(Chain);
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apS = 
/*#__PURE__*/
apS_(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = 
/*#__PURE__*/
of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = 
/*#__PURE__*/
tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/
apT_(Apply);
