import { map as map_, mapLeftDefault } from './Bifunctor';
import { identity, pipe } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const tuple2 = (a, e) => [a, e];
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const fst = (t) => t[0];
/**
 * @since 3.0.0
 */
export const snd = (t) => t[1];
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = (t) => [snd(t), fst(t)];
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const bimap = (mapSnd, mapFst) => (t) => [
    mapFst(fst(t)),
    mapSnd(snd(t))
];
/**
 * Map a function over the second component of a `Tuple2`.
 *
 * This is the `mapLeft` operation of the `Bifunctor` instance.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapSnd = 
/*#__PURE__*/
mapLeftDefault(bimap);
/**
 * @category Semigroupoid
 * @since 3.0.0
 */
export const compose = (bc) => (ab) => [fst(bc), snd(ab)];
/**
 * @category Extend
 * @since 3.0.0
 */
export const extend = (f) => (wa) => [f(wa), snd(wa)];
/**
 * @category Extract
 * @since 3.0.0
 */
export const extract = fst;
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const duplicate = 
/*#__PURE__*/
extend(identity);
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => f(b, fst(fa));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = () => (f) => (fa) => f(fst(fa));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => f(fst(fa), b);
/**
 * @since 3.0.0
 */
export const traverse = (F) => (f) => (t) => pipe(f(fst(t)), F.map((b) => [b, snd(t)]));
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft: mapSnd
};
/**
 * Map a function over the first component of a `Tuple2`.
 *
 * This is the `map` operation of the `Functor` instance.
 *
 * @category Functor
 * @since 3.0.0
 */
export const mapFst = 
/*#__PURE__*/
map_(Bifunctor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map: mapFst
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Semigroupoid = {
    compose
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map: mapFst,
    extend,
    extract
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    map: mapFst,
    traverse
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: (a) => [a, M.empty]
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map: mapFst,
    ap: (fa) => (fab) => [fst(fab)(fst(fa)), S.concat(snd(fa))(snd(fab))]
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    const P = getPointed(M);
    return {
        map: mapFst,
        ap: A.ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => {
    return {
        map: mapFst,
        chain: (f) => (ma) => {
            const [b, s] = f(fst(ma));
            return [b, S.concat(s)(snd(ma))];
        }
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getChain(M);
    return {
        map: mapFst,
        chain: C.chain,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export function getChainRec(M) {
    const chainRec = (f) => (a) => {
        let result = f(a);
        let acc = M.empty;
        let s = fst(result);
        while (_.isLeft(s)) {
            acc = M.concat(snd(result))(acc);
            result = f(s.left);
            s = fst(result);
        }
        return [s.right, M.concat(snd(result))(acc)];
    };
    return {
        chainRec
    };
}
