import { flap as flap_ } from './Functor';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export const tell = (w) => () => [undefined, w];
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const listen = (fa) => () => {
    const [a, w] = fa();
    return [[a, w], w];
};
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const pass = (fa) => () => {
    const [[a, f], w] = fa();
    return [a, f(w)];
};
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export const listens = (f) => (fa) => () => {
    const [a, w] = fa();
    return [[a, f(w)], w];
};
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export const censor = (f) => (fa) => () => {
    const [a, w] = fa();
    return [a, f(w)];
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => () => {
    const [a, w] = fa();
    return [f(a), w];
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: (a) => () => [a, M.empty]
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (fab) => () => {
        const [f, w1] = fab();
        const [a, w2] = fa();
        return [f(a), S.concat(w2)(w1)];
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    const P = getPointed(M);
    return {
        map,
        ap: A.ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => {
    return {
        map,
        chain: (f) => (ma) => () => {
            const [a, w1] = ma();
            const [b, w2] = f(a)();
            return [b, S.concat(w2)(w1)];
        }
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getChain(M);
    return {
        map,
        of: P.of,
        chain: C.chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/
flap_(Functor);
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const evaluate = (fa) => fa()[0];
/**
 * @since 3.0.0
 */
export const execute = (fa) => fa()[1];
