// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getBooleanAlgebra = (BA) => () => ({
    meet: (second) => (first) => (a) => BA.meet(second(a))(first(a)),
    join: (second) => (first) => (a) => BA.join(second(a))(first(a)),
    zero: () => BA.zero,
    one: () => BA.one,
    implies: (second) => (first) => (a) => BA.implies(second(a))(first(a)),
    not: (x) => (a) => BA.not(x(a))
});
/**
 * Unary functions form a semigroup as long as you can provide a semigroup for the codomain.
 *
 * @example
 * import { Predicate } from 'fp-ts/Predicate'
 * import { pipe, getSemigroup } from 'fp-ts/function'
 * import * as B from 'fp-ts/boolean'
 *
 * const f: Predicate<number> = (n) => n <= 2
 * const g: Predicate<number> = (n) => n >= 0
 *
 * const S1 = getSemigroup(B.SemigroupAll)<number>()
 *
 * assert.deepStrictEqual(pipe(f, S1.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, S1.concat(g))(3), false)
 *
 * const S2 = getSemigroup(B.SemigroupAny)<number>()
 *
 * assert.deepStrictEqual(pipe(f, S2.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, S2.concat(g))(3), true)
 *
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = (S) => () => ({
    concat: (second) => (first) => (a) => S.concat(second(a))(first(a))
});
/**
 * Unary functions form a monoid as long as you can provide a monoid for the codomain.
 *
 * @example
 * import { Predicate } from 'fp-ts/Predicate'
 * import { getMonoid, pipe } from 'fp-ts/function'
 * import * as B from 'fp-ts/boolean'
 *
 * const f: Predicate<number> = (n) => n <= 2
 * const g: Predicate<number> = (n) => n >= 0
 *
 * const M1 = getMonoid(B.MonoidAll)<number>()
 *
 * assert.deepStrictEqual(pipe(f, M1.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, M1.concat(g))(3), false)
 *
 * const M2 = getMonoid(B.MonoidAny)<number>()
 *
 * assert.deepStrictEqual(pipe(f, M2.concat(g))(1), true)
 * assert.deepStrictEqual(pipe(f, M2.concat(g))(3), true)
 *
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = (M) => {
    const getSemigroupM = getSemigroup(M);
    return () => ({
        concat: getSemigroupM().concat,
        empty: () => M.empty
    });
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getSemiring = (S) => ({
    add: (second) => (first) => (x) => S.add(second(x))(first(x)),
    zero: () => S.zero,
    mul: (second) => (first) => (x) => S.mul(second(x))(first(x)),
    one: () => S.one
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getRing = (R) => {
    const S = getSemiring(R);
    return {
        add: S.add,
        mul: S.mul,
        one: S.one,
        zero: S.zero,
        sub: (second) => (first) => (x) => R.sub(second(x))(first(x))
    };
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const apply = (a) => (f) => f(a);
/**
 * @since 3.0.0
 */
export const identity = (a) => a;
/**
 * @since 3.0.0
 */
export const unsafeCoerce = identity;
/**
 * @since 3.0.0
 */
export const constant = (a) => () => a;
/**
 * A thunk that returns always `true`.
 *
 * @since 3.0.0
 */
export const constTrue = 
/*#__PURE__*/
constant(true);
/**
 * A thunk that returns always `false`.
 *
 * @since 3.0.0
 */
export const constFalse = 
/*#__PURE__*/
constant(false);
/**
 * A thunk that returns always `null`.
 *
 * @since 3.0.0
 */
export const constNull = 
/*#__PURE__*/
constant(null);
/**
 * A thunk that returns always `undefined`.
 *
 * @since 3.0.0
 */
export const constUndefined = 
/*#__PURE__*/
constant(undefined);
/**
 * A thunk that returns always `void`.
 *
 * @since 3.0.0
 */
export const constVoid = constUndefined;
/**
 * Flips the order of the arguments of a function of two arguments.
 *
 * @since 3.0.0
 */
export const flip = (f) => (b, a) => f(a, b);
export function flow(ab, bc, cd, de, ef, fg, gh, hi, ij) {
    switch (arguments.length) {
        case 1:
            return ab;
        case 2:
            return function () {
                return bc(ab.apply(this, arguments));
            };
        case 3:
            return function () {
                return cd(bc(ab.apply(this, arguments)));
            };
        case 4:
            return function () {
                return de(cd(bc(ab.apply(this, arguments))));
            };
        case 5:
            return function () {
                return ef(de(cd(bc(ab.apply(this, arguments)))));
            };
        case 6:
            return function () {
                return fg(ef(de(cd(bc(ab.apply(this, arguments))))));
            };
        case 7:
            return function () {
                return gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))));
            };
        case 8:
            return function () {
                return hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments))))))));
            };
        case 9:
            return function () {
                return ij(hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))))));
            };
    }
    return;
}
/**
 * @since 3.0.0
 */
export const increment = (n) => n + 1;
/**
 * @since 3.0.0
 */
export const decrement = (n) => n - 1;
/**
 * @since 3.0.0
 */
export const absurd = (_) => {
    throw new Error('Called `absurd` function which should be uncallable');
};
/**
 * Creates a tupled version of this function: instead of `n` arguments, it accepts a single tuple argument.
 *
 * @example
 * import { tupled } from 'fp-ts/function'
 *
 * const add = tupled((x: number, y: number): number => x + y)
 *
 * assert.strictEqual(add([1, 2]), 3)
 *
 * @since 3.0.0
 */
export const tupled = (f) => (a) => f(...a);
/**
 * Inverse function of `tupled`
 *
 * @since 3.0.0
 */
export const untupled = (f) => (...a) => f(a);
export function pipe(a, ab, bc, cd, de, ef, fg, gh, hi) {
    switch (arguments.length) {
        case 1:
            return a;
        case 2:
            return ab(a);
        case 3:
            return bc(ab(a));
        case 4:
            return cd(bc(ab(a)));
        case 5:
            return de(cd(bc(ab(a))));
        case 6:
            return ef(de(cd(bc(ab(a)))));
        case 7:
            return fg(ef(de(cd(bc(ab(a))))));
        case 8:
            return gh(fg(ef(de(cd(bc(ab(a)))))));
        case 9:
            return hi(gh(fg(ef(de(cd(bc(ab(a))))))));
        default:
            let ret = arguments[0];
            for (let i = 1; i < arguments.length; i++) {
                ret = arguments[i](ret);
            }
            return ret;
    }
}
/**
 * Type hole simulation
 *
 * @since 3.0.0
 */
export const hole = absurd;
/**
 * `SK` function (SKI combinator calculus).
 *
 * @since 3.0.0
 */
export const SK = (_, b) => b;
