/**
 * The `Const` type constructor, which wraps its first type argument and ignores its second.
 * That is, `Const<E, A>` is isomorphic to `E` for any `A`.
 *
 * `Const` has some useful instances. For example, the `Applicative` instance allows us to collect results using a `Monoid`
 * while ignoring return values.
 *
 * @since 3.0.0
 */
import type { Applicative2C } from './Applicative'
import type { Apply2C } from './Apply'
import { Bifunctor2 } from './Bifunctor'
import type { BooleanAlgebra } from './BooleanAlgebra'
import type { Bounded } from './Bounded'
import type { Contravariant2 } from './Contravariant'
import type { Eq } from './Eq'
import { Functor2 } from './Functor'
import type { HeytingAlgebra } from './HeytingAlgebra'
import type { Monoid } from './Monoid'
import type { Ord } from './Ord'
import type { Ring } from './Ring'
import type { Semigroup } from './Semigroup'
import type { Semiring } from './Semiring'
import type { Show } from './Show'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Const<E, A> = E & {
  readonly _A: A
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const make: <E, A = never>(e: E) => Const<E, A>
/**
 * @category Contravariant
 * @since 3.0.0
 */
export declare const contramap: Contravariant2<URI>['contramap']
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const bimap: Bifunctor2<URI>['bimap']
/**
 * Map a function over the first type argument of a bifunctor.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: Bifunctor2<URI>['mapLeft']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Const'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Const: Const<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <E, A>(S: Show<E>) => Show<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <E, A>(E: Eq<E>) => Eq<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getOrd: <E, A>(O: Ord<E>) => Ord<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getBounded: <E, A>(B: Bounded<E>) => Bounded<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getSemigroup: <E, A>(S: Semigroup<E>) => Semigroup<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <E, A>(M: Monoid<E>) => Monoid<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getSemiring: <E, A>(S: Semiring<E>) => Semiring<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getRing: <E, A>(S: Ring<E>) => Ring<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getHeytingAlgebra: <E, A>(H: HeytingAlgebra<E>) => HeytingAlgebra<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getBooleanAlgebra: <E, A>(H: BooleanAlgebra<E>) => BooleanAlgebra<Const<E, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Contravariant: Contravariant2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor2<URI>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Const<E, (a: A) => B>) => Const<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <E>(S: Semigroup<E>) => Apply2C<'Const', E>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <E>(M: Monoid<E>) => Applicative2C<'Const', E>
