/**
 * The `Eq` type class represents types which support decidable equality.
 *
 * Instances must satisfy the following laws:
 *
 * 1. Reflexivity: `a |> equals(a) === true`
 * 2. Symmetry: `a |> equals(b) === b |> equals(a)`
 * 3. Transitivity: if `a |> equals(b) === true` and `b |> equals(c) === true`, then `a |> equals(c) === true`
 *
 * @since 3.0.0
 */
import type { Contravariant1 } from './Contravariant'
import type { Monoid } from './Monoid'
import type { Ord } from './Ord'
import type { Semigroup } from './Semigroup'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Eq<A> {
  readonly equals: (second: A) => (first: A) => boolean
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromEquals: <A>(equals: (second: A) => (first: A) => boolean) => Eq<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromOrd: <A>(O: Ord<A>) => Eq<A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const struct: <A>(eqs: { [K in keyof A]: Eq<A[K]> }) => Eq<{ readonly [K_1 in keyof A]: A[K_1] }>
/**
 * Given a tuple of `Eq`s returns a `Eq` for the tuple
 *
 * @example
 * import { tuple } from 'fp-ts/Eq'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import * as B from 'fp-ts/boolean'
 *
 * const E = tuple(S.Eq, N.Eq, B.Eq)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, true]), true)
 * assert.strictEqual(E.equals(['a', 1, true])(['b', 1, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 2, true]), false)
 * assert.strictEqual(E.equals(['a', 1, true])(['a', 1, false]), false)
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tuple: <A extends readonly unknown[]>(
  ...eqs: { [K in keyof A]: Eq<A[K]> }
) => Eq<Readonly<Readonly<A>>>
/**
 * @category Contravariant
 * @since 3.0.0
 */
export declare const contramap: Contravariant1<URI>['contramap']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Eq'
declare module './HKT' {
  interface URItoKind<A> {
    readonly Eq: Eq<A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const EqStrict: Eq<unknown>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getSemigroup: <A>() => Semigroup<Eq<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <A>() => Monoid<Eq<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Contravariant: Contravariant1<URI>
