/**
 * ```ts
 * type Option<A> = None | Some<A>
 * ```
 *
 * `Option<A>` is a container for an optional value of type `A`. If the value of type `A` is present, the `Option<A>` is
 * an instance of `Some<A>`, containing the present value of type `A`. If the value is absent, the `Option<A>` is an
 * instance of `None`.
 *
 * An option could be looked at as a collection or foldable structure with either one or zero elements.
 * Another way to look at `Option` is: it represents the effect of a possibly failing computation.
 *
 * @since 3.0.0
 */
import type { Alt1 } from './Alt'
import type { Alternative1 } from './Alternative'
import type { Applicative1 } from './Applicative'
import { Apply1 } from './Apply'
import { Chain1 } from './Chain'
import type { Compactable1 } from './Compactable'
import type { Either } from './Either'
import { Eq } from './Eq'
import type { Extend1 } from './Extend'
import type { Filterable1 } from './Filterable'
import type { Foldable1 } from './Foldable'
import { FromEither1 } from './FromEither'
import { Lazy } from './function'
import { Functor1 } from './Functor'
import type { Monad1 } from './Monad'
import type { Monoid } from './Monoid'
import { Ord } from './Ord'
import type { Pointed1 } from './Pointed'
import type { Predicate } from './Predicate'
import { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Semigroup } from './Semigroup'
import type { Show } from './Show'
import type { Traversable1 } from './Traversable'
import { Witherable1 } from './Witherable'
import { Zero1 } from './Zero'
/**
 * @category model
 * @since 3.0.0
 */
export interface None {
  readonly _tag: 'None'
}
/**
 * @category model
 * @since 3.0.0
 */
export interface Some<A> {
  readonly _tag: 'Some'
  readonly value: A
}
/**
 * @category model
 * @since 3.0.0
 */
export declare type Option<A> = None | Some<A>
/**
 * Returns `true` if the option is `None`, `false` otherwise.
 *
 * @example
 * import { some, none, isNone } from 'fp-ts/Option'
 *
 * assert.strictEqual(isNone(some(1)), false)
 * assert.strictEqual(isNone(none), true)
 *
 * @category guards
 * @since 3.0.0
 */
export declare const isNone: (fa: Option<unknown>) => fa is None
/**
 * Returns `true` if the option is an instance of `Some`, `false` otherwise.
 *
 * @example
 * import { some, none, isSome } from 'fp-ts/Option'
 *
 * assert.strictEqual(isSome(some(1)), true)
 * assert.strictEqual(isSome(none), false)
 *
 * @category guards
 * @since 3.0.0
 */
export declare const isSome: <A>(fa: Option<A>) => fa is Some<A>
/**
 * `None` doesn't have a constructor, instead you can use it directly as a value. Represents a missing value.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const none: Option<never>
/**
 * Constructs a `Some`. Represents an optional value that exists.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const some: <A>(a: A) => Option<A>
/**
 * Returns the `Left` value of an `Either` if possible.
 *
 * @example
 * import { getLeft, none, some } from 'fp-ts/Option'
 * import { right, left } from 'fp-ts/Either'
 *
 * assert.deepStrictEqual(getLeft(right(1)), none)
 * assert.deepStrictEqual(getLeft(left('a')), some('a'))
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const getLeft: <E, A>(ma: Either<E, A>) => Option<E>
/**
 * Returns the `Right` value of an `Either` if possible.
 *
 * @example
 * import { getRight, none, some } from 'fp-ts/Option'
 * import { right, left } from 'fp-ts/Either'
 *
 * assert.deepStrictEqual(getRight(right(1)), some(1))
 * assert.deepStrictEqual(getRight(left('a')), none)
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const getRight: <E, A>(ma: Either<E, A>) => Option<A>
/**
 * Transforms an `Either` to an `Option` discarding the error.
 *
 * Alias of [getRight](#getRight)
 *
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: FromEither1<URI>['fromEither']
/**
 * Less strict version of [`match`](#match).
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const matchW: <B, A, C>(onNone: Lazy<B>, onSome: (a: A) => C) => (ma: Option<A>) => B | C
/**
 * Takes a (lazy) default value, a function, and an `Option` value, if the `Option` value is `None` the default value is
 * returned, otherwise the function is applied to the value inside the `Some` and the result is returned.
 *
 * @example
 * import { some, none, match } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     match(() => 'a none', a => `a some containing ${a}`)
 *   ),
 *   'a some containing 1'
 * )
 *
 * assert.strictEqual(
 *   pipe(
 *     none,
 *     match(() => 'a none', a => `a some containing ${a}`)
 *   ),
 *   'a none'
 * )
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <B, A>(onNone: Lazy<B>, onSome: (a: A) => B) => (ma: Option<A>) => B
/**
 * Less strict version of [`getOrElse`](#getOrElse).
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseW: <B>(onNone: Lazy<B>) => <A>(ma: Option<A>) => B | A
/**
 * Extracts the value out of the structure, if it exists. Otherwise returns the given default value
 *
 * @example
 * import { some, none, getOrElse } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     getOrElse(() => 0)
 *   ),
 *   1
 * )
 * assert.strictEqual(
 *   pipe(
 *     none,
 *     getOrElse(() => 0)
 *   ),
 *   0
 * )
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <A>(onNone: Lazy<A>) => (ma: Option<A>) => A
/**
 * Transforms an exception into an `Option`. If `f` throws, returns `None`, otherwise returns the output wrapped in a
 * `Some`.
 *
 * See also [`tryCatchK`](#tryCatchK).
 *
 * @example
 * import { none, some, tryCatch } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(
 *   tryCatch(() => {
 *     throw new Error()
 *   }),
 *   none
 * )
 * assert.deepStrictEqual(tryCatch(() => 1), some(1))
 *
 * @category interop
 * @since 3.0.0
 */
export declare const tryCatch: <A>(f: Lazy<A>) => Option<A>
/**
 * Converts a function that may throw to one returning a `Option`.
 *
 * @category interop
 * @since 3.0.0
 */
export declare const tryCatchK: <A extends readonly unknown[], B>(f: (...a: A) => B) => (...a: A) => Option<B>
/**
 * Constructs a new `Option` from a nullable type. If the value is `null` or `undefined`, returns `None`, otherwise
 * returns the value wrapped in a `Some`.
 *
 * @example
 * import { none, some, fromNullable } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(fromNullable(undefined), none)
 * assert.deepStrictEqual(fromNullable(null), none)
 * assert.deepStrictEqual(fromNullable(1), some(1))
 *
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <A>(a: A) => Option<NonNullable<A>>
/**
 * Returns a *smart constructor* from a function that returns a nullable value.
 *
 * @example
 * import { fromNullableK, none, some } from 'fp-ts/Option'
 *
 * const f = (s: string): number | undefined => {
 *   const n = parseFloat(s)
 *   return isNaN(n) ? undefined : n
 * }
 *
 * const g = fromNullableK(f)
 *
 * assert.deepStrictEqual(g('1'), some(1))
 * assert.deepStrictEqual(g('a'), none)
 *
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <A extends readonly unknown[], B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => Option<NonNullable<B>>
/**
 * This is `chain` + `fromNullable`, useful when working with optional values.
 *
 * @example
 * import { some, none, fromNullable, chainNullableK } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * interface Employee {
 *   company?: {
 *     address?: {
 *       street?: {
 *         name?: string
 *       }
 *     }
 *   }
 * }
 *
 * const employee1: Employee = { company: { address: { street: { name: 'high street' } } } }
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     fromNullable(employee1.company),
 *     chainNullableK(company => company.address),
 *     chainNullableK(address => address.street),
 *     chainNullableK(street => street.name)
 *   ),
 *   some('high street')
 * )
 *
 * const employee2: Employee = { company: { address: { street: {} } } }
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     fromNullable(employee2.company),
 *     chainNullableK(company => company.address),
 *     chainNullableK(address => address.street),
 *     chainNullableK(street => street.name)
 *   ),
 *   none
 * )
 *
 * @category interop
 * @since 3.0.0
 */
export declare const chainNullableK: <A, B>(
  f: (a: A) => B | null | undefined
) => (ma: Option<A>) => Option<NonNullable<B>>
/**
 * Extracts the value out of the structure, if it exists. Otherwise returns `null`.
 *
 * @example
 * import { some, none, toNullable } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     toNullable
 *   ),
 *   1
 * )
 * assert.strictEqual(
 *   pipe(
 *     none,
 *     toNullable
 *   ),
 *   null
 * )
 *
 * @category interop
 * @since 3.0.0
 */
export declare const toNullable: <A>(ma: Option<A>) => A | null
/**
 * Extracts the value out of the structure, if it exists. Otherwise returns `undefined`.
 *
 * @example
 * import { some, none, toUndefined } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     toUndefined
 *   ),
 *   1
 * )
 * assert.strictEqual(
 *   pipe(
 *     none,
 *     toUndefined
 *   ),
 *   undefined
 * )
 *
 * @category interop
 * @since 3.0.0
 */
export declare const toUndefined: <A>(ma: Option<A>) => A | undefined
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor1<URI>['map']
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: Apply1<URI>['ap']
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: Pointed1<URI>['of']
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: Chain1<URI>['chain']
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Option<Option<A>>) => Option<A>
/**
 * Less strict version of [`alt`](#alt).
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const altW: <B>(second: Lazy<Option<B>>) => <A>(first: Option<A>) => Option<B | A>
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * In case of `Option` returns the left-most non-`None` value.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     O.some('a'),
 *     O.alt(() => O.some('b'))
 *   ),
 *   O.some('a')
 * )
 * assert.deepStrictEqual(
 *   pipe(
 *     O.none,
 *     O.alt(() => O.some('b'))
 *   ),
 *   O.some('b')
 * )
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: Alt1<URI>['alt']
/**
 * @category Zero
 * @since 3.0.0
 */
export declare const zero: Zero1<URI>['zero']
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: Extend1<URI>['extend']
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(ma: Option<A>) => Option<Option<A>>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: Foldable1<URI>['reduce']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: Foldable1<URI>['foldMap']
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: Foldable1<URI>['reduceRight']
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: Compactable1<URI>['compact']
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: Compactable1<URI>['separate']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filter: Filterable1<URI>['filter']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: Filterable1<URI>['filterMap']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partition: Filterable1<URI>['partition']
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: Filterable1<URI>['partitionMap']
/**
 * @category Traversable
 * @since 3.0.0
 */
export declare const traverse: Traversable1<URI>['traverse']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Option'
declare module './HKT' {
  interface URItoKind<A> {
    readonly Option: Option<A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Option<A>>
/**
 * @example
 * import { none, some, getEq } from 'fp-ts/Option'
 * import * as N from 'fp-ts/number'
 *
 * const E = getEq(N.Eq)
 * assert.strictEqual(E.equals(none)(none), true)
 * assert.strictEqual(E.equals(none)(some(1)), false)
 * assert.strictEqual(E.equals(some(1))(none), false)
 * assert.strictEqual(E.equals(some(1))(some(2)), false)
 * assert.strictEqual(E.equals(some(1))(some(1)), true)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Option<A>>
/**
 * The `Ord` instance allows `Option` values to be compared with
 * `compare`, whenever there is an `Ord` instance for
 * the type the `Option` contains.
 *
 * `None` is considered to be less than any `Some` value.
 *
 *
 * @example
 * import { none, some, getOrd } from 'fp-ts/Option'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const O = getOrd(N.Ord)
 * assert.strictEqual(pipe(none, O.compare(none)), 0)
 * assert.strictEqual(pipe(none, O.compare(some(1))), -1)
 * assert.strictEqual(pipe(some(1), O.compare(none)), 1)
 * assert.strictEqual(pipe(some(1), O.compare(some(2))), -1)
 * assert.strictEqual(pipe(some(1), O.compare(some(1))), 0)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getOrd: <A>(O: Ord<A>) => Ord<Option<A>>
/**
 * Monoid returning the left-most non-`None` value. If both operands are `Some`s then the inner values are
 * concatenated using the provided `Semigroup`
 *
 * | x       | y       | concat(y)(x)       |
 * | ------- | ------- | ------------------ |
 * | none    | none    | none               |
 * | some(a) | none    | some(a)            |
 * | none    | some(a) | some(a)            |
 * | some(a) | some(b) | some(concat(b)(a)) |
 *
 * @example
 * import { getMonoid, some, none } from 'fp-ts/Option'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const M = getMonoid(N.SemigroupSum)
 * assert.deepStrictEqual(pipe(none, M.concat(none)), none)
 * assert.deepStrictEqual(pipe(some(1), M.concat(none)), some(1))
 * assert.deepStrictEqual(pipe(none, M.concat(some(1))), some(1))
 * assert.deepStrictEqual(pipe(some(1), M.concat(some(2))), some(3))
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <A>(S: Semigroup<A>) => Monoid<Option<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor1<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Option<(a: A) => B>) => Option<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply1<URI>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: Option<B>) => <A>(first: Option<A>) => Option<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: Option<B>) => <A>(first: Option<A>) => Option<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad1<URI>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => Option<B>) => (first: Option<A>) => Option<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: Alt1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Zero: Zero1<URI>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const guard: (b: boolean) => Option<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alternative: Alternative1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Extend: Extend1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: Compactable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: Filterable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: Traversable1<URI>
/**
 * @category Witherable
 * @since 3.0.0
 */
export declare const wither: Witherable1<URI>['wither']
/**
 * @category Witherable
 * @since 3.0.0
 */
export declare const wilt: Witherable1<URI>['wilt']
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Witherable: Witherable1<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: FromEither1<URI>
/**
 * Returns a *smart constructor* based on the given predicate.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 *
 * const getOption = O.fromPredicate((n: number) => n >= 0)
 *
 * assert.deepStrictEqual(getOption(-1), O.none)
 * assert.deepStrictEqual(getOption(1), O.some(1))
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <A, B extends A>(refinement: import('./Refinement').Refinement<A, B>): (a: A) => Option<B>
  <A_1>(predicate: Predicate<A_1>): <B_1 extends A_1>(b: B_1) => Option<B_1>
  <A_2>(predicate: Predicate<A_2>): (a: A_2) => Option<A_2>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends readonly unknown[], E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => Option<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: Option<A>) => Option<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstEitherK: <E, A, B>(f: (a: A) => Either<E, B>) => (ma: Option<A>) => Option<A>
/**
 * Tests whether a value is a member of a `Option`.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(pipe(O.some(1), O.elem(N.Eq)(1)), true)
 * assert.strictEqual(pipe(O.some(1), O.elem(N.Eq)(2)), false)
 * assert.strictEqual(pipe(O.none, O.elem(N.Eq)(1)), false)
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: Eq<A>) => (a: A) => (ma: Option<A>) => boolean
/**
 * Returns `true` if the predicate is satisfied by the wrapped value
 *
 * @example
 * import { some, none, exists } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     exists(n => n > 0)
 *   ),
 *   true
 * )
 * assert.strictEqual(
 *   pipe(
 *     some(1),
 *     exists(n => n > 1)
 *   ),
 *   false
 * )
 * assert.strictEqual(
 *   pipe(
 *     none,
 *     exists(n => n > 0)
 *   ),
 *   false
 * )
 *
 * @since 3.0.0
 */
export declare const exists: <A>(predicate: Predicate<A>) => (ma: Option<A>) => boolean
/**
 * @since 3.0.0
 */
export declare const Do: Option<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(name: N) => <A>(fa: Option<A>) => Option<{ [K in N]: A }>
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Option<B>
) => (ma: Option<A>) => Option<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: Option<B>
) => (fa: Option<A>) => Option<{ readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 3.0.0
 */
export declare const ApT: Option<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: Option<A>) => Option<readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <B>(
  fb: Option<B>
) => <A extends readonly unknown[]>(fas: Option<A>) => Option<readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => Option<B>
) => (as: ReadonlyNonEmptyArray<A>) => Option<ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => Option<B>
) => (as: readonly A[]) => Option<readonly B[]>
