/**
 * @since 3.0.0
 */
import type { Applicative2 } from './Applicative'
import { Apply2 } from './Apply'
import type { Category2 } from './Category'
import { Chain2 } from './Chain'
import type { Choice2 } from './Choice'
import { Functor2 } from './Functor'
import type { Monad2 } from './Monad'
import type { Pointed2 } from './Pointed'
import type { Profunctor2 } from './Profunctor'
import type { Semigroupoid2 } from './Semigroupoid'
import type { Strong2 } from './Strong'
import type { FromReader2 } from './FromReader'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface Reader<R, A> {
  (r: R): A
}
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const ask: <R>() => Reader<R, R>
/**
 * Projects a value from the global context in a `Reader`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asks: <R, A>(f: (r: R) => A) => Reader<R, A>
/**
 * Less strict version of [`asksReaderK`](#asksreaderk).
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReaderW: <R1, R2, A>(f: (r1: R1) => Reader<R2, A>) => Reader<R1 & R2, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReader: <R, A>(f: (r: R) => Reader<R, A>) => Reader<R, A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(f: (r2: R2) => R1) => <A>(ma: Reader<R1, A>) => Reader<R2, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * Less strict version of [`ap`](#ap).
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const apW: <R2, A>(fa: Reader<R2, A>) => <R1, B>(fab: Reader<R1, (a: A) => B>) => Reader<R1 & R2, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: Apply2<URI>['ap']
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: Pointed2<URI>['of']
/**
 * Less strict version of [`chain`](#chain).
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chainW: <A, R2, B>(f: (a: A) => Reader<R2, B>) => <R1>(ma: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: Chain2<URI>['chain']
/**
 * Less strict version of [`flatten`](#flatten).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flattenW: <R1, R2, A>(mma: Reader<R1, Reader<R2, A>>) => Reader<R1 & R2, A>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <R, A>(mma: Reader<R, Reader<R, A>>) => Reader<R, A>
/**
 * @category Semigroupoid
 * @since 3.0.0
 */
export declare const compose: Semigroupoid2<URI>['compose']
/**
 * @category Profunctor
 * @since 3.0.0
 */
export declare const promap: Profunctor2<URI>['promap']
/**
 * @category Category
 * @since 3.0.0
 */
export declare const id: Category2<URI>['id']
/**
 * @category Choice
 * @since 3.0.0
 */
export declare const left: Choice2<URI>['left']
/**
 * @category Choice
 * @since 3.0.0
 */
export declare const right: Choice2<URI>['right']
/**
 * @category Strong
 * @since 3.0.0
 */
export declare const first: Strong2<URI>['first']
/**
 * @category Strong
 * @since 3.0.0
 */
export declare const second: Strong2<URI>['second']
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'Reader'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly Reader: Reader<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromReader: FromReader2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: Reader<E, (a: A) => B>) => Reader<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply2<URI>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <E, B>(second: Reader<E, B>) => <A>(first: Reader<E, A>) => Reader<E, A>
/**
 * Less strict version of [`apFirst`](#apfirst).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apFirstW: <R2, A, B>(second: Reader<R2, B>) => <R1>(first: Reader<R1, A>) => Reader<R1 & R2, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <E, B>(second: Reader<E, B>) => <A>(first: Reader<E, A>) => Reader<E, B>
/**
 * Less strict version of [`apSecond`](#apsecond).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apSecondW: <R2, A, B>(second: Reader<R2, B>) => <R1>(first: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad2<URI>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, E, B>(f: (a: A) => Reader<E, B>) => (first: Reader<E, A>) => Reader<E, A>
/**
 * Less strict version of [`chainFirst`](#chainfirst).
 *
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstW: <R2, A, B>(
  f: (a: A) => Reader<R2, B>
) => <R1>(ma: Reader<R1, A>) => Reader<R1 & R2, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Profunctor: Profunctor2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Category: Category2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Choice: Choice2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Strong: Strong2<URI>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <E, A>(fa: Reader<E, A>) => Reader<E, { readonly [K in N]: A }>
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Reader<E, B>
) => (ma: Reader<E, A>) => Reader<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * Less strict version of [`bind`](#bind).
 *
 * @since 3.0.0
 */
export declare const bindW: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Reader<R2, B>
) => <R1>(
  fa: Reader<R1, A>
) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const Do: Reader<unknown, {}>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  fb: Reader<E, B>
) => (fa: Reader<E, A>) => Reader<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * Less strict version of [`apS`](#apS).
 *
 * @since 3.0.0
 */
export declare const apSW: <A, N extends string, R2, B>(
  name: Exclude<N, keyof A>,
  fb: Reader<R2, B>
) => <R1>(
  fa: Reader<R1, A>
) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: Reader<unknown, readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <E, A>(fa: Reader<E, A>) => Reader<E, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <E, B>(
  fb: Reader<E, B>
) => <A extends readonly unknown[]>(fas: Reader<E, A>) => Reader<E, readonly [...A, B]>
/**
 * Less strict version of [`apT`](#apT).
 *
 * @since 3.0.0
 */
export declare const apTW: <R2, B>(
  fb: Reader<R2, B>
) => <R1, A extends ReadonlyArray<unknown>>(fas: Reader<R1, A>) => Reader<R1 & R2, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: ReadonlyNonEmptyArray<A>) => Reader<R, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: readonly A[]) => Reader<R, readonly B[]>
