/**
 * If a type `A` can form a `Semigroup` it has an **associative** binary operation.
 *
 * ```ts
 * interface Semigroup<A> {
 *   readonly concat: (second: A) => (first: A) => A
 * }
 * ```
 *
 * Associativity means the following equality must hold for any choice of `x`, `y`, and `z`.
 *
 * ```ts
 * (x |> concat(y)) |> concat(z) <-> x |> concat(y |> concat(z))
 * ```
 *
 * @since 3.0.0
 */
import type { Endomorphism } from './Endomorphism'
import * as M from './Magma'
import * as O from './Ord'
import Magma = M.Magma
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Semigroup<A> extends Magma<A> {}
/**
 * Get a semigroup where `concat` will return the minimum, based on the provided order.
 *
 * @example
 * import { min } from 'fp-ts/Semigroup'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const S = min(N.Ord)
 *
 * assert.deepStrictEqual(pipe(1, S.concat(2)), 1)
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const min: <A>(o: O.Ord<A>) => Semigroup<A>
/**
 * Get a semigroup where `concat` will return the maximum, based on the provided order.
 *
 * @example
 * import { max } from 'fp-ts/Semigroup'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * const S = max(N.Ord)
 *
 * assert.deepStrictEqual(pipe(1, S.concat(2)), 2)
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const max: <A>(o: O.Ord<A>) => Semigroup<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const constant: <A>(a: A) => Semigroup<A>
/**
 * The dual of a `Semigroup`, obtained by swapping the arguments of `concat`.
 *
 * @example
 * import { reverse } from 'fp-ts/Semigroup'
 * import * as S from 'fp-ts/string'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe('a', reverse(S.Semigroup).concat('b')), 'ba')
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const reverse: <A>(S: Semigroup<A>) => Semigroup<A>
/**
 * Given a struct of semigroups returns a semigroup for the struct.
 *
 * @example
 * import { struct } from 'fp-ts/Semigroup'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * interface Point {
 *   readonly x: number
 *   readonly y: number
 * }
 *
 * const S = struct<Point>({
 *   x: N.SemigroupSum,
 *   y: N.SemigroupSum
 * })
 *
 * assert.deepStrictEqual(pipe({ x: 1, y: 2 }, S.concat({ x: 3, y: 4 })), { x: 4, y: 6 })
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const struct: <A>(
  semigroups: { [K in keyof A]: Semigroup<A[K]> }
) => Semigroup<{ readonly [K_1 in keyof A]: A[K_1] }>
/**
 * Given a tuple of semigroups returns a semigroup for the tuple.
 *
 * @example
 * import { tuple } from 'fp-ts/Semigroup'
 * import { pipe } from 'fp-ts/function'
 * import * as B from 'fp-ts/boolean'
 * import * as N from 'fp-ts/number'
 * import * as S from 'fp-ts/string'
 *
 * const S1 = tuple(S.Semigroup, N.SemigroupSum)
 * assert.deepStrictEqual(pipe(['a', 1], S1.concat(['b', 2])), ['ab', 3])
 *
 * const S2 = tuple(S.Semigroup, N.SemigroupSum, B.SemigroupAll)
 * assert.deepStrictEqual(pipe(['a', 1, true], S2.concat(['b', 2, false])), ['ab', 3, false])
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tuple: <A extends readonly unknown[]>(
  ...semigroups: { [K in keyof A]: Semigroup<A[K]> }
) => Semigroup<Readonly<A>>
/**
 * You can glue items between and stay associative.
 *
 * @example
 * import { intercalate } from 'fp-ts/Semigroup'
 * import { pipe } from 'fp-ts/function'
 * import * as S from 'fp-ts/string'
 *
 * const S1 = pipe(S.Semigroup, intercalate(' + '))
 *
 * assert.strictEqual(pipe('a', S1.concat('b')), 'a + b')
 * assert.strictEqual(pipe('a', S1.concat('b'), S1.concat('c')), 'a + b + c')
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const intercalate: <A>(middle: A) => Endomorphism<Semigroup<A>>
/**
 * Always return the first argument.
 *
 * @example
 * import * as S from 'fp-ts/Semigroup'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(1, S.first<number>().concat(2)), 1)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const first: <A = never>() => Semigroup<A>
/**
 * Always return the last argument.
 *
 * @example
 * import * as S from 'fp-ts/Semigroup'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(1, S.last<number>().concat(2)), 2)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const last: <A = never>() => Semigroup<A>
/**
 * Given a sequence of `as`, concat them and return the total.
 *
 * If `as` is empty, return the provided `startWith` value.
 *
 * @example
 * import { concatAll } from 'fp-ts/Semigroup'
 * import * as N from 'fp-ts/number'
 *
 * const sum = concatAll(N.SemigroupSum)(0)
 *
 * assert.deepStrictEqual(sum([1, 2, 3]), 6)
 * assert.deepStrictEqual(sum([]), 0)
 *
 * @since 3.0.0
 */
export declare const concatAll: <A>(S: Semigroup<A>) => (startWith: A) => (as: ReadonlyArray<A>) => A
