/**
 * @since 3.0.0
 */
import type { Applicative2 } from './Applicative'
import { Apply2 } from './Apply'
import { Chain2 } from './Chain'
import type { Endomorphism } from './Endomorphism'
import type { FromState2 } from './FromState'
import { Functor2 } from './Functor'
import type { Monad2 } from './Monad'
import type { Pointed2 } from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface State<S, A> {
  (s: S): readonly [A, S]
}
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const get: <S>() => State<S, S>
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const put: <S>(s: S) => State<S, void>
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const modify: <S>(f: Endomorphism<S>) => State<S, void>
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const gets: <S, A>(f: (s: S) => A) => State<S, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: Functor2<URI>['map']
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: Apply2<URI>['ap']
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: Pointed2<URI>['of']
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: Chain2<URI>['chain']
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <E, A>(mma: State<E, State<E, A>>) => State<E, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'State'
declare module './HKT' {
  interface URItoKind2<E, A> {
    readonly State: State<E, A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor2<URI>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <E, B>(fab: State<E, (a: A) => B>) => State<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply2<URI>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <E, B>(second: State<E, B>) => <A>(first: State<E, A>) => State<E, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <E, B>(second: State<E, B>) => <A>(first: State<E, A>) => State<E, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain2<URI>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad2<URI>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, E, B>(f: (a: A) => State<E, B>) => (first: State<E, A>) => State<E, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromState: FromState2<URI>
/**
 * Run a computation in the `State` monad, discarding the final state.
 *
 * @since 3.0.0
 */
export declare const evaluate: <S>(s: S) => <A>(ma: State<S, A>) => A
/**
 * Run a computation in the `State` monad discarding the result.
 *
 * @since 3.0.0
 */
export declare const execute: <S>(s: S) => <A>(ma: State<S, A>) => S
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <E, A>(fa: State<E, A>) => State<E, { readonly [K in N]: A }>
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => State<E, B>
) => (ma: State<E, A>) => State<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, E, B>(
  name: Exclude<N, keyof A>,
  fb: State<E, B>
) => (fa: State<E, A>) => State<E, { readonly [K in N | keyof A]: K extends keyof A ? A[K] : B }>
/**
 * @since 3.0.0
 */
export declare const tupled: <E, A>(fa: State<E, A>) => State<E, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <E, B>(
  fb: State<E, B>
) => <A extends readonly unknown[]>(fas: State<E, A>) => State<E, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: ReadonlyNonEmptyArray<A>) => State<S, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: readonly A[]) => State<S, readonly B[]>
